/*
 * $Id: IRemoteTestRunListener.java,v 1.1 2005/02/14 14:36:47 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 02.02.2005 11:37:07 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * COMMENTME Comment type IDDRemoteTestListener
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IRemoteTestRunListener extends Remote {
    /**
     * Status constant indicating that a test passed (constant value 0).
     * 
     * @see #testFailed(int, String, String, String)
     */
    public static final int STATUS_OK = 0;
    /**
     * Status constant indicating that a test had an error an unanticipated
     * exception (constant value 1).
     * 
     * @see #testFailed(int, String, String, String)
     */
    public static final int STATUS_ERROR = 1;
    /**
     * Status constant indicating that a test failed an assertion (constant
     * value 2).
     * 
     * @see #testFailed(int, String, String, String)
     */
    public static final int STATUS_FAILURE = 2;

    /**
     * A test run has started.
     * 
     * @param testCount
     *            the number of individual tests that will be run
     */
    public void testRunStarted(int testCount) throws RemoteException;

    /**
     * A test run has ended.
     * 
     * @param elapsedTime
     *            the total elapsed time of the test run
     */
    public void testRunEnded(long elapsedTime) throws RemoteException;

    /**
     * A test run has been stopped prematurely.
     * 
     * @param elapsedTime
     *            the time elapsed before the test run was stopped
     */
    public void testRunStopped(long elapsedTime) throws RemoteException;

    /**
     * An individual test has started.
     * 
     * @param testId
     *            a unique Id identifying the test
     * @param testName
     *            the name of the test that started
     */
    public void testStarted(String testId, String testName)
            throws RemoteException;

    /**
     * An individual test has ended.
     * 
     * @param testId
     *            a unique Id identifying the test
     * @param testName
     *            the name of the test that ended
     */
    public void testEnded(String testId, String testName)
            throws RemoteException;

    /**
     * An individual test has failed with a stack trace.
     * 
     * @param testId
     *            a unique Id identifying the test
     * @param testName
     *            the name of the test that failed
     * @param status
     *            the outcome of the test; one of
     *            {@link #STATUS_ERROR STATUS_ERROR}or
     *            {@link #STATUS_FAILURE STATUS_FAILURE}
     * @param trace
     *            the stack trace
     */
    public void testFailed(int status, String testId, String testName,
            String trace) throws RemoteException;

    /**
     * The VM instance performing the tests has terminated.
     */
    public void testRunTerminated() throws RemoteException;

    /**
     * An individual test has been rerun.
     * 
     * @param testId
     *            a unique Id identifying the test
     * @param testClass
     *            the name of the test class that was rerun
     * @param testName
     *            the name of the test that was rerun
     * @param status
     *            the outcome of the test that was rerun; one of
     *            {@link #STATUS_OK STATUS_OK},
     *            {@link #STATUS_ERROR STATUS_ERROR}, or
     *            {@link #STATUS_FAILURE STATUS_FAILURE}
     * @param trace
     *            the stack trace in the case of abnormal termination, or the
     *            empty string if none
     */
    public void testReran(String testId, String testClass, String testName,
            int status, String trace) throws RemoteException;

}