/*
 * $Id: DDResultCache.java,v 1.1 2005/08/14 12:52:56 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University Prof. Dr. Andreas Zeller
 * 
 * All rights reserved. This program and the accompanying materials are made
 * available under the terms of the Common Public License v1.0 which accompanies
 * this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Created on 16.07.2005 13:32:25 by @author Karsten Lehmann Mail to:
 * lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client.cts;

import java.util.HashMap;
import java.util.Map;

import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;

/**
 * 
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
class DDResultCache {

	/** The cache for the outcome */
	private Map fCache = new HashMap();

	/**
	 * Add the given key and dd result to the cache if it doesn't contain.
	 * 
	 * @param key
	 *            the key you want to add to the cache for the dd result.
	 * @param ddResult
	 *            the delta debbuging result for the key.
	 */
	public void addDDResult(Object key, IDDRunResult ddResult) {
		if (!this.fCache.containsKey(key)) {
			this.fCache.put(key, ddResult);
		}
	}

	/**
	 * Check if the delta debugging result of the given key is already cache. If
	 * it do so return true, otherwise false.
	 * 
	 * @param key
	 *            the key you look for.
	 * @return true if the delta debugging result of the given key is in the
	 *         ache, otherwise false.
	 */
	public boolean existDDResultForKey(Object key) {
		boolean result = false;

		result = this.fCache.containsKey(key);

		return result;
	}

	/**
	 * Returns the cached delta debugging result of the given key.
	 * 
	 * @param key
	 *            the key for which you want to get the delta debugging result.
	 * @throws IllegalArgumentException
	 *             if the delta debugging result of the given delta set is not
	 *             in cache.
	 * @return the cached delta debugging result of the given key.
	 */
	public IDDRunResult getDDResult(Object key) {
		IDDRunResult result = null;

		if (!this.existDDResultForKey(key))
			throw new IllegalArgumentException(
					"No outcome exitst for DeltaSet\n" + key);

		result = (IDDRunResult) this.fCache.get(key);

		return result;
	}

	/**
	 * Removes all mappings between the keys and delta debugging results from
	 * this map.
	 */
	public void clear() {
		this.fCache.clear();
	}
}