/*
 * $Id: ICTSRunListenerManager.java,v 1.2 2005/08/17 17:27:05 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 17, 2005 6:06:54 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client.cts;

import org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription;

/**
 * This manager notify all registered listener about changes in the cause transition
 * run. At this listener you can add and remove your cause transition listener.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface ICTSRunListenerManager {

	/**
	 * Registers the given listener for cts run notifications. Has no effect if
	 * an identical listener is already registered.
	 * 
	 * @param listener
	 *            listener to register
	 */
	public void addCTSRunListener(ICTSRunListener listener);

	/**
	 * Deregisters the given listener for cts run notifications. Has no effect if
	 * an identical listener is not already registered.
	 * 
	 * @param listener
	 *            listener to deregister
	 */
	public void removeCTSRunListener(ICTSRunListener listener);

	/**
	 * Returns a collection of cts run listeners registered with this cts run
	 * listener manager.
	 * 
	 * @return a collection of cts run listeners registered with this cts run
	 *         listener manager
	 */
	public ICTSRunListener[] getCTSRunListeners();

	/**
	 * Notify the listeners that a new cause transition run was started. For a
	 * specific description of the cts run look at the given parameter object.
	 * 
	 * @param desc
	 *            the description with all informations to the cts run.
	 */
	public void notifyCTSRunStarted(IDDLaunchDescription desc);

	/**
	 * Notify the listeners that the cause transition run has terminated.
	 * 
	 * @param result
	 *            the result of the cause transition run.
	 */
	public void notifyCTSRunTerminated(ICTSRunResult result);
}