/*
 * $Id: JUnitTestDescription.java,v 1.1 2005/08/10 14:33:14 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.02.2005 14:26:07 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.client.testrunner;

import java.io.Serializable;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;


/**
 * A JUnit test description includes the test case class name and the test
 * method name.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class JUnitTestDescription implements IJUnitTestDescription,
        Serializable {

	private static final long serialVersionUID = -6109444767977514191L;

	/** full qualified class name of the test case */
    private final String fTestCaseClassName;

    /** the name of the test method within the test case */
    private final String fTestName;

    /**
     * Create a new JUnit test description.
     * 
     * @param testCaseClassName
     *            the full qualified class name of the test case.
     * @param testName
     *            the name of the test method within the test case.
     */
    public JUnitTestDescription(String testCaseClassName, String testName) {
        this.fTestCaseClassName = testCaseClassName;
        this.fTestName = testName;
    }

    /**
     * @see org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription#getTestCaseClassName()
     */
    public String getTestCaseClassName() {
        return this.fTestCaseClassName;
    }

    /**
     * @see org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription#getTestName()
     */
    public String getTestName() {
        return this.fTestName;
    }

    public String toString() {
        return "Suite: " + this.fTestCaseClassName + " - Test: "
                + this.fTestName;
    }
}