/*
 * $Id: DDLaunchDescription.java,v 1.3 2005/08/11 08:15:52 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.06.2005 08:45:10 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.launching.IVMInstall;

/**
 * The launch description includes all necessary informations to launch a delta
 * debugging run.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDLaunchDescription implements IDDLaunchDescription {

	private int fDDLaunchID = -1;

	private final IJavaProject fJavaProject;

	private final IVMInstall fVMInstall;

	private final IMatchPoint[] fMatchPoints;

	private final IJUnitTestDescription fPassJunitTestDesc;

	private final IJUnitTestDescription fFailJunitTestDesc;

	public DDLaunchDescription(IJavaProject javaProject, IVMInstall vmInstall,
			IJUnitTestDescription passJunitTestDesc,
			IJUnitTestDescription failJunitTestDesc) {
		this(javaProject, vmInstall, passJunitTestDesc, failJunitTestDesc,
				(IMatchPoint[]) null);
	}

	public DDLaunchDescription(IJavaProject javaProject, IVMInstall vmInstall,
			IJUnitTestDescription passJunitTestDesc,
			IJUnitTestDescription failJunitTestDesc, IMatchPoint matchPoint) {
		this(javaProject, vmInstall, passJunitTestDesc, failJunitTestDesc,
				new IMatchPoint[] { matchPoint });
	}

	public DDLaunchDescription(IJavaProject javaProject, IVMInstall vmInstall,
			IJUnitTestDescription passJunitTestDesc,
			IJUnitTestDescription failJunitTestDesc, IMatchPoint[] matchPoints) {

		this.fJavaProject = javaProject;
		this.fVMInstall = vmInstall;

		this.fPassJunitTestDesc = passJunitTestDesc;
		this.fFailJunitTestDesc = failJunitTestDesc;

		this.fMatchPoints = matchPoints;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getJavaProject()
	 */
	public IJavaProject getJavaProject() {
		return this.fJavaProject;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getVMInstall()
	 */
	public IVMInstall getVMInstall() {
		return this.fVMInstall;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getPassJUnitTestDescription()
	 */
	public IJUnitTestDescription getPassJUnitTestDescription() {
		return this.fPassJunitTestDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getFailJUnitTestDescription()
	 */
	public IJUnitTestDescription getFailJUnitTestDescription() {
		return this.fFailJunitTestDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getDDLaunchID()
	 */
	public int getDDLaunchID() {
		return this.fDDLaunchID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#setDDLaunchID(int)
	 */
	public void setDDLaunchID(int launchID) {
		this.fDDLaunchID = launchID;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription#getMatchPoints()
	 */
	public IMatchPoint[] getMatchPoints() {
		return this.fMatchPoints;
	}

}