/*
 * $Id: IDDRunListener.java,v 1.3 2005/08/16 09:14:11 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 10.04.2005 17:09:28 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ddrun;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.memorygraph.IRootNode;

/**
 * A dd run listener receives notifications of the process of a delta debugging
 * run.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public interface IDDRunListener {

	/**
	 * Notify the listeners that a new delta debugging run was started. For a
	 * specific description of the dd run look at the given parameter object.
	 * 
	 * @param desc
	 *            the description with all informations to run delta debugging.
	 */
	public void ddRunStarted(IDDRunDescription desc);

	/**
	 * Notify the listeners that the memory graph of the passing run and the
	 * memory graph of the failing run are extracted.
	 * 
	 * @param ddRunID
	 *            the unique id of the delta debugging run.
	 * @param passGraph
	 *            the passing graph, which was extracted.
	 * @param failGraph
	 *            the failing graph, which was extracted.
	 */
	public void memoryGraphsExtracted(int ddRunID, IRootNode passGraph,
			IRootNode failGraph);

	/**
	 * Notify the listeners that the memory graphs was compared with each other.
	 * 
	 * @param ddRunID
	 *            the unique id of the delta debugging run.
	 * @param passGraph
	 *            the compared passing graph.
	 * @param failGraph
	 *            the compared failing graph.
	 * @param deltas
	 *            the deltas computed between passing and failing graph.
	 */
	public void memoryGraphsCompared(int ddRunID, IRootNode passGraph,
			IRootNode failGraph, DeltaSet deltas);

	/**
	 * Notify the listeners that the delta debugging run has terminated.
	 * 
	 * @param result
	 *            the result of the delta debugging run.
	 */
	public void ddRunTerminated(IDDRunResult result);

}
