/*
 * $Id: MatchPoint.java,v 1.4 2005/08/21 11:43:26 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Jul 20, 2005 1:43:19 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.matcher;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.IBreakpointLocation;
import org.deltadebugging.ddstate.logger.DDLogger;

/**
 * A match point is a pair of two breakpoints. A match point determine a
 * comparable location within two test excecutions. Comparable in the sense that
 * at the defined location we can apply delta debugging.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class MatchPoint implements IMatchPoint {

	/** Personal logger for this class */
	final static DDLogger logger = DDStateLaunchPlugin
			.getLogger(MatchPoint.class);

	private final IBreakpointLocation fPassingBreakpointDesc;

	private final IBreakpointLocation fFailingBreakpointDesc;

	public MatchPoint(IBreakpointLocation passingBreakpointDesc,
			IBreakpointLocation failingBreakpointDesc) {
		this.fPassingBreakpointDesc = passingBreakpointDesc;
		this.fFailingBreakpointDesc = failingBreakpointDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.matcher.IMatchPoint#getPassingBreakpointLocation()
	 */
	public IBreakpointLocation getPassingBreakpointLocation() {
		return this.fPassingBreakpointDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.matcher.IMatchPoint#getFailingBreakpointLocation()
	 */
	public IBreakpointLocation getFailingBreakpointLocation() {
		return this.fFailingBreakpointDesc;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("Passing breakpoint location:");
		result.append(this.fPassingBreakpointDesc.toString());
		result.append("Failing breakpoint location:");
		result.append(this.fFailingBreakpointDesc.toString());

		return result.toString();
	}

}
