/*
 * $Id: StackTraceRangeComparator.java,v 1.3 2005/07/27 15:19:22 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 21.07.2005 21:06:52 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.matcher;

import java.util.Arrays;

import org.eclipse.compare.rangedifferencer.IRangeComparator;

/**
 * COMMENTME Comment type StackTraceRangeComparator
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class StackTraceRangeComparator implements IRangeComparator {

    private final StackTrace[] fStackTrace;
    private final int[][] fNeutralizedMethodTrace;
    private final int fMethodID;

    public StackTraceRangeComparator(StackTrace[] stackTrace, int methodID) {
        this.fStackTrace = stackTrace;
        this.fMethodID = methodID;
        this.fNeutralizedMethodTrace = new int[this.fStackTrace.length][];
    }

    public StackTrace getStackTrace(int index) {
        return this.fStackTrace[index];
    }

    public int[] getNeutralizedMethodTrace(int index) {
        int[] result = null;

        result = this.fNeutralizedMethodTrace[index];
        if (result == null) {
            result = this.generateNeutralizedMethodTrace(index);
        }

        return result;
    }

    private int[] generateNeutralizedMethodTrace(int index) {
        int[] result = null;

        int[] methodIDTrace = this.getStackTrace(index).getMethodIDTrace();
        result = this.neutraliseMethodID(methodIDTrace, 0);
        this.fNeutralizedMethodTrace[index] = result;

        return result;
    }

    private int[] neutraliseMethodID(int[] methodTrace, int neutralNumber) {
        for (int i = 0; i < methodTrace.length; i++) {
            int j = methodTrace[i];
            if (j == this.fMethodID) {
                methodTrace[i] = neutralNumber;
            }
        }

        return methodTrace;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.compare.rangedifferencer.IRangeComparator#getRangeCount()
     */
    public int getRangeCount() {
        return this.fStackTrace.length;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.compare.rangedifferencer.IRangeComparator#rangesEqual(int,
     *      org.eclipse.compare.rangedifferencer.IRangeComparator, int)
     */
    public boolean rangesEqual(int thisIndex, IRangeComparator other,
            int otherIndex) {

        if (!(other instanceof StackTraceRangeComparator))
            return false;

        StackTraceRangeComparator otherSRC = (StackTraceRangeComparator) other;

        int[] thisMethodIDTrace = this.getNeutralizedMethodTrace(thisIndex);
        int[] otherMethodIDTrace = otherSRC
                .getNeutralizedMethodTrace(otherIndex);

        return Arrays.equals(thisMethodIDTrace, otherMethodIDTrace);
    }

    /**
     * @see org.eclipse.compare.rangedifferencer.IRangeComparator#skipRangeComparison(int,
     *      int, org.eclipse.compare.rangedifferencer.IRangeComparator)
     */
    public boolean skipRangeComparison(int length, int maxLength,
            IRangeComparator other) {
        return false;
    }

}
