/*
 * $Id: RemoteTestRunner.java,v 1.19 2005/09/29 07:15:45 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 12.02.2005 13:20:56 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.client.ObjectCreationException;
import org.deltadebugging.ddstate.launcher.matcher.MethodTrace;
import org.deltadebugging.ddstate.launcher.server.instrumenter.IdentifierMap;
import org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate;

/**
 * COMMENTME Comment type RemoteTestRunner
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class RemoteTestRunner extends UnicastRemoteObject implements
		IRemoteTestRunner {

	static final long serialVersionUID = -3158642513822076402L;

	private JUnitTestRunner fTestRunner = null;

	private boolean fIsRunning = false;

	public RemoteTestRunner(boolean debugMode) throws RemoteException {
		super();
		this.fTestRunner = new JUnitTestRunner(debugMode);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#run(org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription)
	 */
	public IPredicate run(IJUnitTestDescription testDesc)
			throws RemoteException {
		IPredicate result = null;

		this.fIsRunning = true;

		try {
			result = this.fTestRunner.run(testDesc);
		} catch (Throwable t) {
			t.printStackTrace();

		} finally {
			this.fIsRunning = false;

		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#getCurrentClassLoaderID()
	 */
	public int getCurrentClassLoaderID() throws RemoteException {
		return this.fTestRunner.getCurrentClassLoaderID();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#createNewArray(java.lang.String,
	 *      int)
	 */
	public int createNewArray(String className, int length)
			throws ObjectCreationException, RemoteException {
		return this.fTestRunner.createNewArray(className, length);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#createNewClass(java.lang.String)
	 */
	public int createNewClass(String instanceType)
			throws ObjectCreationException, RemoteException {
		return this.fTestRunner.createNewClass(instanceType);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#createNewString(java.lang.String)
	 */
	public int createNewString(String string) throws ObjectCreationException,
			RemoteException {
		return this.fTestRunner.createNewString(string);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#createNewObject(java.lang.String)
	 */
	public int createNewObject(String className)
			throws ObjectCreationException, RemoteException {
		return this.fTestRunner.createNewObject(className);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#isRunning()
	 */
	public boolean isRunning() throws RemoteException {
		return this.fIsRunning;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#loadClass(java.lang.String)
	 */
	public void loadClass(String className) throws RemoteException,
			ClassNotFoundException {
		this.fTestRunner.loadClass(className);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.server.IRemoteTestRunner#generateTrace(org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription,
	 *      org.deltadebugging.ddstate.launcher.server.instrumenter.IdentifierMap)
	 */
	public MethodTrace generateTrace(IJUnitTestDescription testDesc,
			IdentifierMap identifierMap) throws GenerateTraceException,
			RemoteException {
		MethodTrace result = null;

		this.fIsRunning = true;

		try {
			result = this.fTestRunner.generateTrace(testDesc, identifierMap);

		} catch (Throwable t) {
			System.err.println("Error on generating trace");
			t.printStackTrace();

		} finally {
			this.fIsRunning = false;

		}

		return result;
	}

}