/*
 *  
 */
package org.deltadebugging.ddstate.launcher.server.instrumenter;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.NoSuchElementException;

import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.MethodGen;

public class IdentifierMap implements Serializable {

    static final long serialVersionUID = -7208473886661242599L;

    private HashMap fIdentifierMap = null;
    private HashMap fReverseMap = null;
    private int fID = 1;

    public IdentifierMap() {
        this.fIdentifierMap = new HashMap(1000);
        this.fReverseMap = new HashMap(1000);
    }

    public int getMaxID() {
        return this.fID;
    }

    public MethodIdentifier getMethodIdentifier(int methodId) {
        return (MethodIdentifier) this.fReverseMap.get(new Integer(methodId));
    }

    public String getClassName(int classId) {
        return (String) this.fReverseMap.get(new Integer(classId));
    }

    public boolean isClassIdentifier(int id) {
        return this.fReverseMap.get(new Integer(id)) instanceof String;
    }

    public boolean isMethodIdentifier(int id) {
        return this.fReverseMap.get(new Integer(id)) instanceof MethodIdentifier;
    }

    public int getIdentifier(MethodIdentifier methodIdentifier)
            throws NoSuchElementException {
        return lookupIdentifier(methodIdentifier);
    }
    
    public boolean existsIdentifierFor(MethodIdentifier methodIdentifier) {
        boolean result = false;
        
        result = this.fIdentifierMap.containsKey(methodIdentifier);
        
        return result;
    }

    public int getIdentifier(String className, String methodName,
            String signature) throws NoSuchElementException {
        return lookupIdentifier(new MethodIdentifier(className, methodName,
                signature));
    }
    
    public boolean existsIdentifierFor(String className, String methodName,
            String signature) {
        return this.existsIdentifierFor(new MethodIdentifier(className, methodName,
                signature));
    }

    public int getIdentifier(JavaClass javaClass, MethodGen methodGenerator) {
        return getIdentifier(javaClass.getClassName(), methodGenerator
                .getName(), methodGenerator.getSignature());
    }

    public int getIdentifier(JavaClass javaClass, Method method) {
        return getIdentifier(javaClass.getClassName(), method.getName(), method
                .getSignature());
    }

    public int getIdentifier(String className) throws NoSuchElementException {
        return lookupIdentifier(className);
    }

    private int lookupIdentifier(Object key) throws NoSuchElementException {
        int result = -1;

        Integer id = (Integer) this.fIdentifierMap.get(key);

        if (id != null) {
            result = id.intValue();

        } else {
            // extend the lookup method. if the key not exists put it into the
            // map
            result = this.fID++;
            id = new Integer(result);

            this.fIdentifierMap.put(key, id);
            this.fReverseMap.put(id, key);
        }

        return result;
    }

    public String toString() {
        Iterator scanner;
        StringBuffer result;
        Object key;

        result = new StringBuffer();
        scanner = this.fIdentifierMap.keySet().iterator();
        while (scanner.hasNext()) {
            key = scanner.next();
            result.append(key + ":" + this.fIdentifierMap.get(key) + "\n");
        }
        return result.toString();
    }
}
