/*
 *
 */
package org.deltadebugging.ddstate.launcher.server.instrumenter;

import java.util.NoSuchElementException;

import org.apache.bcel.Constants;
import org.apache.bcel.classfile.JavaClass;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.ClassGen;
import org.apache.bcel.generic.ConstantPoolGen;
import org.apache.bcel.generic.INVOKESPECIAL;
import org.apache.bcel.generic.Instruction;
import org.apache.bcel.generic.InstructionConstants;
import org.apache.bcel.generic.InstructionFactory;
import org.apache.bcel.generic.InstructionHandle;
import org.apache.bcel.generic.InstructionList;
import org.apache.bcel.generic.InvokeInstruction;
import org.apache.bcel.generic.LocalVariableGen;
import org.apache.bcel.generic.MethodGen;
import org.apache.bcel.generic.ObjectType;
import org.apache.bcel.generic.PUSH;
import org.apache.bcel.generic.ReturnInstruction;
import org.apache.bcel.generic.Type;
import org.deltadebugging.ddstate.launcher.bcel.instrumenter.AbstractClassInstrumenter;
import org.softevo.sibrelib.InstrumentationException;
import org.softevo.sibrelib.runtime.util.Utilities;

/**
 * @author dallmeier
 * 
 * TODO To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
public class SimpleMethodTraceInstrumenter extends AbstractClassInstrumenter {

    protected IdentifierMap fIdentifierMap = null;
    protected String fRuntimeClassName = null;

    private static final String NAME_EXCEPTIONVARIABLE = "___exception";

    public SimpleMethodTraceInstrumenter(IdentifierMap identifierMap,
            String runtimeClassName) {
        this(identifierMap, new String[0], runtimeClassName);
    }

    public SimpleMethodTraceInstrumenter(IdentifierMap identifierMap,
            String[] ignoredPackages, String runtimeClassName) {
        super(ignoredPackages);
        this.fIdentifierMap = identifierMap;
        this.fRuntimeClassName = runtimeClassName;
        this.addPackageToIgnore(runtimeClassName);
    }

    protected InstructionList getInlineInstructions(MethodGen methodGenerator,
            JavaClass javaClass, ConstantPoolGen constantPoolGenerator,
            InstructionFactory factory, String methodName)
            throws InstrumentationException {
        InstructionList result;
        int id;

        result = new InstructionList();

        try {
            id = this.fIdentifierMap.getIdentifier(javaClass.getClassName(),
                    methodGenerator.getName(), methodGenerator.getSignature());

            result.append(new PUSH(constantPoolGenerator, id));
        } catch (NoSuchElementException e) {
            throw new InstrumentationException(
                    "Unable to lookup id for method "
                            + javaClass.getClassName() + "#"
                            + methodGenerator.getName() + "#"
                            + methodGenerator.getSignature());
        }

        result.append(factory.createInvoke(this.fRuntimeClassName, methodName,
                Type.VOID, new Type[] { Type.INT }, Constants.INVOKESTATIC));
        return result;
    }

    protected InstructionList getReturnInlineInstructions(
            MethodGen methodGenerator, JavaClass javaClass,
            ConstantPoolGen constantPoolGenerator, InstructionFactory factory)
            throws InstrumentationException {
        return getInlineInstructions(methodGenerator, javaClass,
                constantPoolGenerator, factory, "methodExit");
    }

    protected InstructionList getExceptionHandlerInlineInstructions(
            MethodGen methodGenerator, JavaClass javaClass,
            ConstantPoolGen constantPoolGenerator, InstructionFactory factory)
            throws InstrumentationException {
        return getInlineInstructions(methodGenerator, javaClass,
                constantPoolGenerator, factory, "methodExit");
    }

    protected InstructionList getMethodEnterInstructions(
            MethodGen methodGenerator, JavaClass javaClass,
            ConstantPoolGen constantPoolGenerator, InstructionFactory factory)
            throws InstrumentationException {
        return getInlineInstructions(methodGenerator, javaClass,
                constantPoolGenerator, factory, "methodEnter");
    }

    protected Method instrumentForCallStack(MethodGen methodGenerator,
            JavaClass javaClass, ConstantPoolGen constantPoolGenerator,
            InstructionFactory factory) throws InstrumentationException {
        InstructionList instructionList, inlineInstructions;
        InstructionHandle firstInstruction, lastInstruction, exceptionHandlerStart, exceptionHandlerEnd, inlineCodeStart;
        LocalVariableGen exceptionVariable;
        Method result;
        int instructionCounter = 0;
        InstructionHandle[] instructions;
        Instruction instruction;
        InvokeInstruction invokeInstruction;

        instructionList = methodGenerator.getInstructionList();
        firstInstruction = instructionList.getStart();
        lastInstruction = instructionList.getEnd();
        inlineInstructions = getExceptionHandlerInlineInstructions(
                methodGenerator, javaClass, constantPoolGenerator, factory);
        // if the method that is instrumented is a constructor,
        // we start the exception handler right AFTER the call to
        // the super constructor
        if (methodGenerator.getName().equals("<init>")) {
            instructions = instructionList.getInstructionHandles();
            for (; instructionCounter < instructions.length; instructionCounter++) {
                instruction = instructions[instructionCounter].getInstruction();
                if (instruction instanceof INVOKESPECIAL) {
                    invokeInstruction = (INVOKESPECIAL) instruction;
                    // we found the super class constructor call
                    if (invokeInstruction.getClassName(constantPoolGenerator)
                            .equals(javaClass.getSuperclassName())) {
                        firstInstruction = instructions[instructionCounter + 1];
                        break;
                    }
                }
            }
        }
        exceptionVariable = methodGenerator.addLocalVariable(
                NAME_EXCEPTIONVARIABLE, new ObjectType("java.lang.Throwable"),
                Utilities.getNextVariableIndex(methodGenerator), null, null);
        exceptionHandlerStart = instructionList.append(InstructionFactory
                .createStore(Type.OBJECT, exceptionVariable.getIndex()));
        instructionList.append(inlineInstructions);
        instructionList.append(InstructionFactory.createLoad(Type.OBJECT,
                exceptionVariable.getIndex()));
        exceptionHandlerEnd = instructionList
                .append(InstructionConstants.ATHROW);
        // end of exception handler code
        methodGenerator.addExceptionHandler(firstInstruction, lastInstruction,
                exceptionHandlerStart, null);
        exceptionVariable.setStart(exceptionHandlerStart);
        exceptionVariable.setEnd(exceptionHandlerEnd);
        instructions = instructionList.getInstructionHandles();
        for (int counter = 0; counter < instructions.length; counter++) {
            if (instructions[counter].getInstruction() instanceof ReturnInstruction) {
                // System.out.println("Adding inline after " +
                // instructions[counter].getInstruction());
                inlineInstructions = getReturnInlineInstructions(
                        methodGenerator, javaClass, constantPoolGenerator,
                        factory);
                inlineCodeStart = inlineInstructions.getInstructionHandles()[0];
                instructionList.insert(instructions[counter],
                        inlineInstructions);
                instructionList.redirectBranches(instructions[counter],
                        inlineCodeStart);
            }
        }
        // append the method enter code
        instructionList.insert(getMethodEnterInstructions(methodGenerator,
                javaClass, constantPoolGenerator, factory));
        methodGenerator.setMaxStack();
        methodGenerator.setMaxLocals();
        result = methodGenerator.getMethod();
        // System.out.println(result.getName());
        // System.out.println(result.getCode());
        instructionList.dispose();
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.bcel.instrumenter.AbstractClassInstrumenter#modifyClass(org.apache.bcel.classfile.JavaClass,
     *      org.apache.bcel.generic.ConstantPoolGen,
     *      org.apache.bcel.generic.ClassGen,
     *      org.apache.bcel.generic.InstructionFactory)
     */
    public JavaClass modifyClass(JavaClass clazz,
            ConstantPoolGen constantPoolGenerator, ClassGen classGenerator,
            InstructionFactory factory)
            throws org.deltadebugging.ddstate.launcher.bcel.instrumenter.InstrumentationException {
        Method[] methods;
        MethodGen methodGenerator;
        Method changedMethod;

        try {
            methods = classGenerator.getMethods();
            for (int counter = 0; counter < methods.length; counter++) {
                methodGenerator = new MethodGen(methods[counter], clazz
                        .getClassName(), constantPoolGenerator);
                if (!methodGenerator.isAbstract()
                        && !methodGenerator.isNative()) {

                    // K.L. disregard constructor methods and static methods
                    if (!methodGenerator.getName().equals("<init>")
                            && !methodGenerator.getName().equals("<clinit>")) {
                        changedMethod = instrumentForCallStack(methodGenerator,
                                clazz, constantPoolGenerator, factory);
                        classGenerator.setMethodAt(changedMethod, counter);
                    }
                }
            }
        } catch (InstrumentationException e) {
            throw new org.deltadebugging.ddstate.launcher.bcel.instrumenter.InstrumentationException(
                    e);
        }

        return classGenerator.getJavaClass();
    }
}
