/*
 * $Id: ClassController.java,v 1.2 2005/08/16 15:19:59 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.05.2005 10:00:00 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.server.instrumenter.rtclass;

/**
 * The class controller is used to create an empty object. So if we have loaded
 * and modify the class we want to create we can call a constructor with an
 * argument of the type of this class controller. <br>
 * 
 * E.g. if yo want to create an empty hash map you must call <br>
 * <code>new HashMap(ClassController.instance)</code> <br>
 * 
 * With the global variable <code>WITH_STATIC_BLOCK</code> you can turn the
 * execution of the static block on the first time a class will load on or off.
 * <br>
 * 
 * To prepare the classes for empty class creation you must use the CreateInstanceInstrumenter.
 * 
 * @see org.deltadebugging.ddstate.launcher.server.instrumenter.CreateInstanceInstrumenter
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class ClassController {

	/**
	 * Use this variable to controll the execution of the static block on class
	 * loading.
	 */
	public static boolean WITH_STATIC_BLOCK = true;

	/**
	 * Instance of this class controller
	 */
	public static final ClassController instance = new ClassController();
}
