/*
 * $Id: GenericDDStateTest.java,v 1.12 2005/08/16 15:44:34 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 31.05.2005 18:56:30 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.test;

import junit.framework.TestCase;

import org.deltadebugging.ddstate.launcher.client.IDDLauncher;
import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;
import org.deltadebugging.ddstate.launcher.client.SimpleDDLauncher;
import org.deltadebugging.ddstate.launcher.client.testrunner.JUnitTestDescription;
import org.deltadebugging.ddstate.launcher.ddrun.DDLaunchDescription;
import org.deltadebugging.ddstate.launcher.ddrun.IDDLaunchDescription;
import org.deltadebugging.ddstate.launcher.matcher.IMatchPoint;
import org.deltadebugging.ddstate.launcher.matcher.MatchPoint;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.ILineBreakpointLocation;
import org.deltadebugging.ddstate.launcher.matcher.breakpoint.LineBreakpointLocation;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;

/**
 * COMMENTME Comment type GenericDDStateTest
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GenericDDStateTest extends TestCase {

	IDDLauncher fDDLauncher = null;

	public GenericDDStateTest(String name) {
		super(name);

		this.fDDLauncher = new SimpleDDLauncher();
	}

	protected void launchGenericDDStateTests(String suiteClassName,
			int passBPLineNumber, int failBPLineNumber) throws CoreException {

		IJUnitTestDescription passJUnitTestDesc = new JUnitTestDescription(
				suiteClassName, "testPassingRun");
		IJUnitTestDescription failJUnitTestDesc = new JUnitTestDescription(
				suiteClassName, "testFailingRun");

		ILineBreakpointLocation passLineBPDesc = new LineBreakpointLocation(
				suiteClassName, passBPLineNumber, 1);
		ILineBreakpointLocation failLineBPDesc = new LineBreakpointLocation(
				suiteClassName, failBPLineNumber, 1);

		IMatchPoint matchPoint = new MatchPoint(passLineBPDesc, failLineBPDesc);

		IDDLaunchDescription ddLaunchDesc = new DDLaunchDescription(this
				.getJavaProject(), this.getVMInstall(), passJUnitTestDesc,
				failJUnitTestDesc, matchPoint);

		this.start(ddLaunchDesc);

		try {
			int i = 0;
			while (i < 50) {
				Thread.sleep(1000);
				i++;
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public Job start(final IDDLaunchDescription ddLaunchDesc) {
		Job ddJob = new Job("Deltadebugging launch") {

			protected IStatus run(IProgressMonitor monitor) {

				GenericDDStateTest.this.fDDLauncher
						.start(ddLaunchDesc, monitor);

				return Status.OK_STATUS;
			}

		};

		ddJob.schedule();

		return ddJob;
	}

	public IJavaProject getJavaProject() {
		IJavaProject result = null;

		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject project = root.getProject(DDStateTests.PROJECT_NAME);

		IJavaProject javaProject = JavaCore.create(project);
		if (javaProject != null && javaProject.exists()) {
			result = javaProject;
		}

		return result;
	}

	public IVMInstall getVMInstall() throws CoreException {
		return JavaRuntime.getVMInstall(getJavaProject());
	}

}
