/*
 * $Id: ExceptionPredicate.java,v 1.2 2005/09/13 11:13:46 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 12, 2005 1:52:22 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.tester.predicate;

import junit.framework.TestFailure;

import org.deltadebugging.ddstate.launcher.client.IJUnitTestDescription;

/**
 * The exception predicate defines a predicate for every failed junit test run,
 * which not failed with an AssertionFailure caused by an assert on a value
 * comparation. For this we use the ComparisionPredicate.
 * 
 * @see org.deltadebugging.ddstate.launcher.tester.predicate.ComparisonPredicate
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public class ExceptionPredicate extends FailurePredicate {

	private static final long serialVersionUID = -5374911504023614434L;

	public ExceptionPredicate(TestFailure failure,
			IJUnitTestDescription testDesc) {
		super(failure, testDesc);
	}

	public ExceptionPredicate(Throwable exception,
			IJUnitTestDescription testDesc) {
		super(exception, testDesc);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate#matches(org.deltadebugging.ddstate.launcher.tester.predicate.IPredicate)
	 */
	public boolean matches(IPredicate predicate) {
		boolean result = false;

		if (predicate instanceof ExceptionPredicate) {
			result = super.matches(predicate);
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append("ExceptionPredicate\n");
		result.append("\texception message: " + this.getExceptionMessage()
				+ "\n");
		result.append("\texception trace  : "
				+ this.getExceptionTraceAsString());

		return result.toString();
	}

}
