/*
 * $Id: DDMainTab.java,v 1.5 2006/01/31 18:57:02 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 20.07.2005 19:18:25 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.launcher.ui.tabgroup;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Vector;

import org.deltadebugging.ddstate.launcher.DDStateLaunchPlugin;
import org.deltadebugging.ddstate.launcher.IDDLaunchConstants;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.debug.ui.ILaunchConfigurationTab;
import org.eclipse.jdt.core.Flags;
import org.eclipse.jdt.core.IClassFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.ISourceReference;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
import org.eclipse.jdt.internal.junit.launcher.TestSelectionDialog;
import org.eclipse.jdt.internal.junit.util.TestSearchEngine;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.ui.JavaElementLabelProvider;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.dialogs.SelectionDialog;

/**
 * COMMENTME Comment type DDMainTab
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDMainTab extends AbstractLaunchConfigurationTab implements
        IDDLaunchConstants {

    // Personal logger for this class
    final static DDLogger logger = DDStateLaunchPlugin
            .getLogger(DDMainTab.class);

    private static final String DD_MAIN_TAB_NAME = "Test";

    private static final String ERR_MSG_TEST_NOT_VALID = "Specified class is not a valid test class";
    private static final String ERR_MSG_TEST_NOT_EXISTS = "Test class does not exists";
    private static final String ERR_MSG_TEST_NOT_SPECIFIED = "Test not specified";
    private static final String ERR_MSG_NOT_JAVA_PROJECT = "Specified project is not a Java project";
    private static final String ERR_MSG_PROJECT_NOT_EXISTS = "Project does not exists";
    private static final String ERR_MSG_PROJECT_NOT_SPECIFIED = "Project not specified";
    private static final String ERR_MSG_FAIL_METHOD_NOT_SPECIFIED = "Failing test method not specified";
    private static final String ERR_MSG_PASS_METHOD_NOT_SPECIFIED = "Passing test method not specified";
    private static final String ERR_MSG_EQUAL_TEST_METHODS = "Failing and passing test method must not be equal";

    public final static String TEST_INTERFACE_NAME = "junit.framework.Test";

    private static final String EMPTY_STRING = "";

    Text txtProject;
    private Button butProjectSearch;
    private Text txtTest;
    Button butTestSearch;

    private Combo cmbPassMethodName;
    private Combo cmbFailMethodName;

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        Composite comp = new Composite(parent, SWT.NONE);
        setControl(comp);

        GridLayout topLayout = new GridLayout();
        topLayout.numColumns = 3;
        comp.setLayout(topLayout);

        this.addLabel(comp, "", 0, 3);
        this.createSingleTestSection(comp);

        this.addLabel(comp, "", 0, 3);
        this.createPassTestNameSection(comp);
        this.createFailTestNameSection(comp);

        this.addLabel(comp, "", 0, 3);

        Dialog.applyDialogFont(comp);
        validatePage();
    }

    private void createFailTestNameSection(Composite parent) {
        this.addLabel(parent, "Failing test:", 0);

        this.cmbFailMethodName = new Combo(parent, SWT.DROP_DOWN);
        this.cmbFailMethodName.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.cmbFailMethodName.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });

        this.addLabel(parent, "", 0);
    }

    private void createPassTestNameSection(Composite parent) {
        this.addLabel(parent, "Passing test:", 0);

        this.cmbPassMethodName = new Combo(parent, SWT.DROP_DOWN);
        this.cmbPassMethodName.setLayoutData(new GridData(
                GridData.FILL_HORIZONTAL));
        this.cmbPassMethodName.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
            }
        });

        this.addLabel(parent, "", 0);
    }

    private void createSingleTestSection(Composite parent) {
        this.addLabel(parent, "Project:", 0);

        this.txtProject = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtProject.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        this.txtProject.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
                DDMainTab.this.butTestSearch
                        .setEnabled(DDMainTab.this.txtProject.getText()
                                .length() > 0);
                updateTestNames();
            }
        });

        this.butProjectSearch = new Button(parent, SWT.PUSH);
        this.butProjectSearch.setText("Browse..."); //$NON-NLS-1$
        this.butProjectSearch.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                handleProjectButtonSelected();
            }
        });

        this.addLabel(parent, "Test class:", 0);

        this.txtTest = new Text(parent, SWT.SINGLE | SWT.BORDER);
        this.txtTest.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        this.txtTest.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent evt) {
                updateLaunchConfigurationDialog();
                updateTestNames();
            }
        });

        this.butTestSearch = new Button(parent, SWT.PUSH);
        this.butTestSearch.setEnabled(this.txtProject.getText().length() > 0);
        this.butTestSearch.setText("Search..."); //$NON-NLS-1$
        this.butTestSearch.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                handleSearchButtonSelected();
            }
        });

        new Label(parent, SWT.NONE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#updateLaunchConfigurationDialog()
     */
    protected void updateLaunchConfigurationDialog() {
        validatePage();
        super.updateLaunchConfigurationDialog();
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
     */
    public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
        IJavaElement javaElement = getContext();
        if (javaElement != null) {
            initializeJavaProject(javaElement, configuration);
        } else {
            // We set empty attributes for project & main type so that when one
            // config is
            // compared to another, the existence of empty attributes doesn't
            // cause an
            // incorrect result (the performApply() method can result in empty
            // values
            // for these attributes being set on a config if there is nothing in
            // the
            // corresponding text boxes)
            configuration.setAttribute(
                    IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, ""); //$NON-NLS-1$
        }

        this.initializeTestAttributes(javaElement, configuration);

        this.initializeMatchPointFinder(configuration);
    }

    private void initializeMatchPointFinder(
            ILaunchConfigurationWorkingCopy config) {
        config.setAttribute(ENABLE_MATCHPOINT_FINDER, true);
    }

    private void initializeTestAttributes(IJavaElement javaElement,
            ILaunchConfigurationWorkingCopy config) {
        if (javaElement != null
                && javaElement.getElementType() < IJavaElement.COMPILATION_UNIT)
            config.setAttribute(
                    IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME, "");
        else
            initializeTestType(javaElement, config);
    }

    /**
     * Set the main type & name attributes on the working copy based on the
     * IJavaElement
     */
    private void initializeTestType(IJavaElement javaElement,
            ILaunchConfigurationWorkingCopy config) {
        String name = ""; //$NON-NLS-1$
        try {
            // we only do a search for compilation units or class files or
            // or source references
            if ((javaElement instanceof ICompilationUnit)
                    || (javaElement instanceof ISourceReference)
                    || (javaElement instanceof IClassFile)) {

                IType[] types = TestSearchEngine
                        .findTests(new Object[] { javaElement });
                if ((types == null) || (types.length < 1)) {
                    return;
                }
                // Simply grab the first main type found in the searched element
                name = types[0].getFullyQualifiedName();
            }
        } catch (InterruptedException ie) {
            // do nothing
        } catch (InvocationTargetException ite) {
            // do nothing
        }
        if (name == null)
            name = ""; //$NON-NLS-1$
        config.setAttribute(
                IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME, name);
        initializeName(config, name);
    }

    private void initializeName(ILaunchConfigurationWorkingCopy config,
            String name) {
        if (name == null) {
            name = ""; //$NON-NLS-1$
        }
        if (name.length() > 0) {
            int index = name.lastIndexOf('.');
            if (index > 0) {
                name = name.substring(index + 1);
            }
            name = getLaunchConfigurationDialog().generateName(name);
            config.rename(name);
        }
    }

    /**
     * Set the java project attribute based on the IJavaElement.
     */
    private void initializeJavaProject(IJavaElement javaElement,
            ILaunchConfigurationWorkingCopy config) {
        IJavaProject javaProject = javaElement.getJavaProject();
        String name = null;
        if (javaProject != null && javaProject.exists()) {
            name = javaProject.getElementName();
        }
        config.setAttribute(
                IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, name);
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
     */
    public void initializeFrom(ILaunchConfiguration configuration) {
        this.updateProjectFromConfig(configuration);
        this.updateTestTypeFromConfig(configuration);
        this.updateTestMethodsFromConfig(configuration);
    }

    /**
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
     */
    public void performApply(ILaunchConfigurationWorkingCopy config) {

        config.setAttribute(
                IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME,
                this.txtProject.getText());
        config.setAttribute(
                IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME,
                this.txtTest.getText());

        String passingTestName = this.cmbPassMethodName.getText().trim();
        String failingTestName = this.cmbFailMethodName.getText().trim();

        if (!passingTestName.equals(EMPTY_STRING)) {
            config.setAttribute(PASSING_TEST_NAME, passingTestName);
        }

        if (!failingTestName.equals(EMPTY_STRING)) {
            config.setAttribute(FAILING_TEST_NAME, failingTestName);
        }
    }

    /**
     * @see ILaunchConfigurationTab#isValid(ILaunchConfiguration)
     */
    public boolean isValid(ILaunchConfiguration config) {
        return getErrorMessage() == null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
     */
    public String getName() {
        return DD_MAIN_TAB_NAME;
    }

    private void addLabel(Composite parent, String text, int horizontalIndent) {
        this.addLabel(parent, text, horizontalIndent, 1);
    }

    private void addLabel(Composite parent, String text, int horizontalIndent,
            int horizontalSpan) {
        Label label = new Label(parent, SWT.NONE);
        label.setText(text);
        GridData gd = new GridData(GridData.BEGINNING);
        gd.horizontalIndent = horizontalIndent;
        gd.horizontalSpan = horizontalSpan;
        label.setLayoutData(gd);
    }

    /**
     * Show a dialog that lets the user select a project. This in turn provides
     * context for the main type, allowing the user to key a main type name, or
     * constraining the search for main types to the specified project.
     */
    void handleProjectButtonSelected() {
        IJavaProject project = chooseJavaProject();
        if (project == null) {
            return;
        }

        String projectName = project.getElementName();
        this.txtProject.setText(projectName);
    }

    /**
     * Show a dialog that lists all main types
     */
    void handleSearchButtonSelected() {
        Shell shell = getShell();

        IJavaProject javaProject = getJavaProject();

        IType[] types = new IType[0];
        try {
            // fix for 66922 Wrong radio behaviour when switching
            types = TestSearchEngine.findTests(PlatformUI.getWorkbench()
                    .getProgressService(), new Object[] { javaProject });
        } catch (InterruptedException e) {
            setErrorMessage(e.getMessage());
            return;
        } catch (InvocationTargetException e) {
            logger.log(IStatus.ERROR, "Searching of all tests failed", e
                    .getTargetException());
            return;
        }

        SelectionDialog dialog = new TestSelectionDialog(shell, types);
        dialog.setTitle("Test Selection"); //$NON-NLS-1$
        dialog.setMessage("Choose a test case:"); //$NON-NLS-1$
        if (dialog.open() == Window.CANCEL) {
            return;
        }

        Object[] results = dialog.getResult();
        if ((results == null) || (results.length < 1)) {
            return;
        }
        IType type = (IType) results[0];

        if (type != null) {
            this.txtTest.setText(type.getFullyQualifiedName());
            javaProject = type.getJavaProject();
            this.txtProject.setText(javaProject.getElementName());
        }
    }

    /**
     * Realize a Java Project selection dialog and return the first selected
     * project, or null if there was none.
     */
    protected IJavaProject chooseJavaProject() {
        IJavaProject[] projects;
        try {
            projects = JavaCore.create(getWorkspaceRoot()).getJavaProjects();
        } catch (JavaModelException e) {
            logger.log(e.getStatus());
            projects = new IJavaProject[0];
        }

        ILabelProvider labelProvider = new JavaElementLabelProvider(
                JavaElementLabelProvider.SHOW_DEFAULT);
        ElementListSelectionDialog dialog = new ElementListSelectionDialog(
                getShell(), labelProvider);
        dialog.setTitle("Project Selection"); //$NON-NLS-1$
        dialog
                .setMessage("Choose a project to constrain the search for main types:"); //$NON-NLS-1$
        dialog.setElements(projects);

        IJavaProject javaProject = getJavaProject();
        if (javaProject != null) {
            dialog.setInitialSelections(new Object[] { javaProject });
        }
        if (dialog.open() == Window.OK) {
            return (IJavaProject) dialog.getFirstResult();
        }
        return null;
    }

    /**
     * Return the IJavaProject corresponding to the project name in the project
     * name text field, or null if the text does not match a project name.
     */
    private IJavaProject getJavaProject() {
        String projectName = this.txtProject.getText().trim();
        if (projectName.length() < 1) {
            return null;
        }
        return getJavaModel().getJavaProject(projectName);
    }

    /**
     * Convenience method to get the workspace root.
     */
    private IWorkspaceRoot getWorkspaceRoot() {
        return ResourcesPlugin.getWorkspace().getRoot();
    }

    /**
     * Convenience method to get access to the java model.
     */
    private IJavaModel getJavaModel() {
        return JavaCore.create(getWorkspaceRoot());
    }

    void validatePage() {
        setErrorMessage(null);
        setMessage(null);

        String projectName = this.txtProject.getText().trim();
        if (projectName.length() == 0) {
            setErrorMessage(ERR_MSG_PROJECT_NOT_SPECIFIED); //$NON-NLS-1$
            return;
        }

        IProject project = getWorkspaceRoot().getProject(projectName);
        if (!project.exists()) {
            setErrorMessage(ERR_MSG_PROJECT_NOT_EXISTS); //$NON-NLS-1$
            return;
        }

        try {
            if (!project.hasNature(JavaCore.NATURE_ID)) {
                setErrorMessage(ERR_MSG_NOT_JAVA_PROJECT); //$NON-NLS-1$
                return;
            }
            IJavaProject jProject = getJavaProject();
            String className = this.txtTest.getText().trim();
            if (className.length() == 0) {
                setErrorMessage(ERR_MSG_TEST_NOT_SPECIFIED); //$NON-NLS-1$
                return;
            }
            IType type = jProject.findType(className);
            if (type == null)
                setErrorMessage(ERR_MSG_TEST_NOT_EXISTS); //$NON-NLS-1$
            else if (!DDMainTab.isTestType(type))
                setErrorMessage(ERR_MSG_TEST_NOT_VALID); //$NON-NLS-1$
        } catch (Exception e) {
            // do nothgin
        }

        String passTestMethodName = this.cmbPassMethodName.getText().trim();
        String failTestMethodName = this.cmbFailMethodName.getText().trim();

        if (passTestMethodName.equals(EMPTY_STRING)) {
            setErrorMessage(ERR_MSG_PASS_METHOD_NOT_SPECIFIED);
            return;
        }

        if (failTestMethodName.equals(EMPTY_STRING)) {
            setErrorMessage(ERR_MSG_FAIL_METHOD_NOT_SPECIFIED);
            return;
        }

        if (passTestMethodName.equals(failTestMethodName)) {
            setErrorMessage(ERR_MSG_EQUAL_TEST_METHODS);
            return;
        }
    }

    private static boolean isTestType(IType type) throws JavaModelException {
        if (Flags.isAbstract(type.getFlags()))
            return false;
        if (!Flags.isPublic(type.getFlags()))
            return false;

        IType[] interfaces = type.newSupertypeHierarchy(null)
                .getAllSuperInterfaces(type);
        for (int i = 0; i < interfaces.length; i++)
            if (interfaces[i].getFullyQualifiedName().equals(
                    TEST_INTERFACE_NAME))
                return true;
        return false;
    }

    protected void updateProjectFromConfig(ILaunchConfiguration config) {
        String projectName = ""; //$NON-NLS-1$
        try {
            projectName = config.getAttribute(
                    IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, ""); //$NON-NLS-1$
        } catch (CoreException ce) {
            // do nothing
        }
        this.txtProject.setText(projectName);
    }

    protected void updateTestTypeFromConfig(ILaunchConfiguration config) {
        String testTypeName = ""; //$NON-NLS-1$
        try {
            testTypeName = config.getAttribute(
                    IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME, ""); //$NON-NLS-1$
        } catch (CoreException ce) {
            // do nothgin
        }
        this.txtTest.setText(testTypeName);
    }

    public void activated(ILaunchConfigurationWorkingCopy workingCopy) {
        // do nothing
    }

    protected void updateTestMethodsFromConfig(ILaunchConfiguration config) {
        String passingTestName = null;
        String failingTestName = null;

        //
        // first load the test names
        //
        try {
            passingTestName = config.getAttribute(PASSING_TEST_NAME,
                    EMPTY_STRING);
        } catch (CoreException doNothing) {
            // do nothing
        }

        try {
            failingTestName = config.getAttribute(FAILING_TEST_NAME,
                    EMPTY_STRING);
        } catch (CoreException doNothing) {
            // do nothing
        }

        //
        // load the available test methods
        //
        this.updateTestNames();

        this.cmbPassMethodName.setText(passingTestName);
        this.cmbFailMethodName.setText(failingTestName);
    }

    void updateTestNames() {
        String[] methodNames = this.getTestNames();

        this.cmbFailMethodName.setItems(methodNames);
        this.cmbPassMethodName.setItems(methodNames);
    }

    /**
     * Returns the current Java element context from which to initialize default
     * settings, or <code>null</code> if none.
     * 
     * @return Java element context.
     */
    private IJavaElement getContext() {
        IWorkbenchPage page = JDIDebugUIPlugin.getActivePage();
        if (page != null) {
            ISelection selection = page.getSelection();
            if (selection instanceof IStructuredSelection) {
                IStructuredSelection ss = (IStructuredSelection) selection;
                if (!ss.isEmpty()) {
                    Object obj = ss.getFirstElement();
                    if (obj instanceof IJavaElement) {
                        return (IJavaElement) obj;
                    }
                    if (obj instanceof IResource) {
                        IJavaElement je = JavaCore.create((IResource) obj);
                        if (je == null) {
                            IProject pro = ((IResource) obj).getProject();
                            je = JavaCore.create(pro);
                        }
                        if (je != null) {
                            return je;
                        }
                    }
                }
            }
            IEditorPart part = page.getActiveEditor();
            if (part != null) {
                IEditorInput input = part.getEditorInput();
                return (IJavaElement) input.getAdapter(IJavaElement.class);
            }
        }
        return null;
    }

    public String[] getTestNames() {
        Collection result = new Vector();
        String[] emptyResult = new String[0];

        String typeName = this.txtTest.getText().trim();
        String projectName = this.txtProject.getText().trim();

        if (typeName.equals(EMPTY_STRING) || projectName.equals(EMPTY_STRING))
            return emptyResult;

        //
        // find java project if available
        //
        IJavaProject javaProject = this.getJavaProject();

        if (javaProject == null)
            return emptyResult;

        //
        // find java element in project
        //        
        IType type = null;

        try {
            type = javaProject.findType(typeName);
        } catch (JavaModelException e1) {
            // do nothing
        }

        if (type == null)
            return emptyResult;

        //
        // extract method names
        //
        try {
            IJavaElement javaElements[] = type.getChildren();
            for (int i = 0; i < javaElements.length; i++) {
                IJavaElement element = javaElements[i];
                if (element.getElementType() == IJavaElement.METHOD) {
                    IMethod method = (IMethod) element;
                    // no constructor
                    if (!method.isConstructor()) {

                        // no parameters
                        if (method.getNumberOfParameters() == 0) {

                            // not private and not protected
                            if (!Flags.isPrivate(method.getFlags())
                                    && !Flags.isProtected(method.getFlags())) {

                                // starts with test
                                if (method.getElementName().startsWith("test")) {
                                    result.add(method.getElementName());
                                }
                            }
                        }
                    }
                }
            }
        } catch (JavaModelException e) {
            // do nothing
        }

        return (String[]) result.toArray(new String[result.size()]);
    }
    
    public Image getImage() {
        return DDStateLaunchImages.getImage(DDStateLaunchImages.IMG_KEY_TEST);
    }

}
