/*
 * $Id: GraphDeltaTableViewer.java,v 1.8 2005/11/22 14:23:16 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 19.04.2005 17:06:36 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.views;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.memorygraph.delta.ICreateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IDeleteDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IUpdateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.create.CreateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.delete.DeleteDelta;
import org.deltadebugging.ddstate.memorygraph.delta.delete.DeleteObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateArraySizeDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateObjectReferenceDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateObjectValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateStringDelta;
import org.deltadebugging.ddstate.memorygraph.value.IArrayNode;
import org.deltadebugging.ddstate.memorygraph.value.IClassObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.INullValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IStringObjectNode;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;
import org.deltadebugging.ddstate.ui.images.DDStateUIImages;
import org.deltadebugging.ddstate.ui.images.IDDStateUIImageConstants;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;

/**
 * COMMENTME Comment type GraphDeltaTableViewer
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphDeltaTableViewer extends TableViewer {

    public GraphDeltaTableViewer(Composite parent, int style) {
        super(parent, style);
    }

    public GraphDeltaTableViewer(Table table) {
        super(table);
    }

    public GraphDeltaTableViewer(Composite parent) {
        super(parent);
    }

    {
        this.setContentProvider(new ViewContentProvider());
        this.setLabelProvider(new ViewLabelProvider());
    }

    public void setDeltaSet(DeltaSet deltaSet) {
        if (deltaSet != null) {
            IMemoryGraphDelta[] deltaArray = (IMemoryGraphDelta[]) deltaSet
                    .toArray(new IMemoryGraphDelta[deltaSet.size()]);

            super.setInput(deltaArray);

        }
    }

    class ViewContentProvider implements IStructuredContentProvider {
        IMemoryGraphDelta[] deltas = new IMemoryGraphDelta[0];

        public void inputChanged(Viewer v, Object oldInput, Object newInput) {
            if (newInput instanceof IMemoryGraphDelta[]) {
                this.deltas = (IMemoryGraphDelta[]) newInput;
            }

        }

        public Object[] getElements(Object parent) {
            return this.deltas;
        }

        public void dispose() {
            this.deltas = null;
        }
    }

    class ViewLabelProvider extends LabelProvider implements
            ITableLabelProvider {
        public String getColumnText(Object obj, int index) {
            String result = "No data found"; //$NON-NLS-1$

            IMemoryGraphDelta delta = (IMemoryGraphDelta) obj;

            switch (index) {
            case 0:
                result = getText(delta.getLocation().getLocationAsString());
                break;
            case 1:
                result = getText(GraphDeltaTableViewer
                        .getOldValueAsString(delta));
                break;
            case 2:
                result = getText(GraphDeltaTableViewer
                        .getNewValueAsString(delta));
                break;
            default:
                break;
            }

            return result;
        }

        public Image getColumnImage(Object obj, int index) {

            Image result = null;

            if (index == 0) {

                IMemoryGraphDelta delta = (IMemoryGraphDelta) obj;

                String key = null;
                if (delta instanceof ICreateDelta) {
                    key = IDDStateUIImageConstants.IMG_DELTA_CREATE;

                } else if (delta instanceof IUpdateDelta) {
                    key = IDDStateUIImageConstants.IMG_DELTA_UPDATE;

                } else if (delta instanceof IDeleteDelta) {
                    key = IDDStateUIImageConstants.IMG_DELTA_DELETE;

                }

                result = DDStateUIImages.getImage(key);
            }

            return result;
        }
    }

    /**
     * Returns a string representation for the value before the delta was
     * applied.
     * 
     * @param delta
     * @return a string representation for the value before the delta was
     *         applied.
     */
    public static String getOldValueAsString(IMemoryGraphDelta delta) {
        String result = "No data found"; //$NON-NLS-1$

        if (delta instanceof CreateDelta) {
            result = "";

        } else if (delta instanceof UpdateObjectValueDelta) {
            UpdateObjectValueDelta updateDelta = (UpdateObjectValueDelta) delta;
            IValueNode oldValue = updateDelta.getOldValue();
            
            if (delta instanceof UpdateArraySizeDelta) {
                result = "" + ((IArrayNode) oldValue).getLength();
                
            } else if (delta instanceof UpdatePrimitiveValueDelta) {
                result = ((IPrimitiveValueNode) oldValue).getStringValue();
                
            } else if (delta instanceof UpdateStringDelta) {
                result = ((IStringObjectNode) oldValue).getStringValue();
                
            }

        } else if (delta instanceof DeleteDelta) {

            if (delta instanceof DeleteObjectReferenceDelta) {
                DeleteObjectReferenceDelta deleteDelta = (DeleteObjectReferenceDelta) delta;

                result = "instance of "
                        + ((IValueNode) deleteDelta.getVariableToDelete()
                                .getTargetNode()).getShortReferenceTypeName();
            }

        } else if (delta instanceof UpdateObjectReferenceDelta) {
            result = "";

        }

        return result;
    }

    /**
     * Returns a string representation for the value after the delta was
     * applied.
     * 
     * @param delta
     * @return a string representation for the value after the delta was
     *         applied.
     */
    public static String getNewValueAsString(IMemoryGraphDelta delta) {
        String result = "No data found"; //$NON-NLS-1$

        if (delta instanceof CreateDelta) {
            IValueNode valueNode = ((CreateDelta) delta).getValueToCreate();

            if (valueNode instanceof INullValueNode) {
                // this should never happen

            } else if (valueNode instanceof IPrimitiveValueNode) {
                result = ((IPrimitiveValueNode) valueNode).getStringValue();

            } else {
                result = "instance of "
                        + ((IObjectNode) valueNode).getShortReferenceTypeName();

                if (valueNode instanceof IStringObjectNode) {
                    result += " (\""
                            + ((IStringObjectNode) valueNode).getStringValue()
                            + "\")";
                }

                if (valueNode instanceof IClassObjectNode) {
                    result += " (type="
                            + ((IClassObjectNode) valueNode).getInstanceType()
                            + ")";
                }
            }

        } else if (delta instanceof UpdateObjectValueDelta) {
            UpdateObjectValueDelta updateDelta = (UpdateObjectValueDelta) delta;
            IValueNode newValue = updateDelta.getNewValue();
            
            if (delta instanceof UpdateArraySizeDelta) {
                result = "" + ((IArrayNode) newValue).getLength();
                
            } else if (delta instanceof UpdatePrimitiveValueDelta) {
                result = ((IPrimitiveValueNode) newValue).getStringValue();
                
            } else if (delta instanceof UpdateStringDelta) {
                result = ((IStringObjectNode) newValue).getStringValue();
                
            }

        } else if (delta instanceof DeleteDelta) {
            result = "";

        } else if (delta instanceof UpdateObjectReferenceDelta) {
            result = ((UpdateObjectReferenceDelta) delta).getNewLocation()
                    .getLocationAsString();

        }

        return result;
    }

}