/*
 * $Id: DDRunDropDownAction.java,v 1.1 2005/04/20 09:15:47 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 09.04.2005 16:43:22 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.views.actions;

import java.util.List;

import org.deltadebugging.ddstate.launcher.ddrun.IDDRunResult;
import org.deltadebugging.ddstate.ui.DDStateUIPlugin;
import org.deltadebugging.ddstate.ui.images.DDStateUIImages;
import org.deltadebugging.ddstate.ui.images.IDDStateUIImageConstants;
import org.deltadebugging.ddstate.ui.resultmanager.IDDRunResultListener;
import org.deltadebugging.ddstate.ui.views.DDRunResultView;
import org.deltadebugging.ddstate.ui.views.IDDRunResultView;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;

/**
 * COMMENTME Comment type DDRunDropDownAction
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class DDRunDropDownAction extends Action implements IMenuCreator,
        IDDRunResultListener {

    Menu fMenu = null;
    IDDRunResultView fView = null;

    /**
     *  
     */
    public DDRunDropDownAction(IDDRunResultView view) {
        this.fView = view;
        this.setText("Delta debugging run results"); //$NON-NLS-1$
        this.setToolTipText("Delta debugging run results"); //$NON-NLS-1$

        setImageDescriptor(DDStateUIImages
                .getImageDescriptor(IDDStateUIImageConstants.IMG_ELCL_VIEW));
        setDisabledImageDescriptor(DDStateUIImages
                .getImageDescriptor(IDDStateUIImageConstants.IMG_ELCL_VIEW));
        setMenuCreator(this);
        DDStateUIPlugin.getDefault().getDDRunResultManager()
                .addDDRunResultListener(this);
        update();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Control)
     */
    public Menu getMenu(Control parent) {
        if (this.fMenu != null) {
            this.fMenu.dispose();
        }

        this.fMenu = new Menu(parent);
        IDDRunResult[] ddRunResults = DDStateUIPlugin.getDefault()
                .getDDRunResultManager().getDDRunResults();
        IDDRunResult current = this.fView.getDDRunResult();

        for (int i = 0; i < ddRunResults.length; i++) {
            IDDRunResult ddRunResult = ddRunResults[i];

            Action action = new ShowDDRunResultAction(this.fView, ddRunResult);
            action.setChecked(ddRunResult.equals(current));
            this.addActionToMenu(this.fMenu, action);
        }

        return this.fMenu;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Menu)
     */
    public Menu getMenu(Menu parent) {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.IMenuCreator#dispose()
     */
    public void dispose() {
        if (this.fMenu != null) {
            this.fMenu.dispose();
        }
    }

    void update() {
        IDDRunResult[] resultes = DDStateUIPlugin.getDefault()
                .getDDRunResultManager().getDDRunResults();

        this.setEnabled(resultes.length > 1);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultListener#ddRunResultsAdded(org.deltadebugging.ddstate.ui.views.IDDRunResult[])
     */
    public void ddRunResultsAdded(IDDRunResult[] results) {
        Display display = DDStateUIPlugin.getStandardDisplay();
        display.asyncExec(new Runnable() {
            public void run() {
                update();
            }
        });

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.ui.views.IDDRunResultListener#ddRunResultsRemoved(org.deltadebugging.ddstate.ui.views.IDDRunResult[])
     */
    public void ddRunResultsRemoved(IDDRunResult[] results) {
        Display display = DDStateUIPlugin.getStandardDisplay();
        display.asyncExec(new Runnable() {
            public void run() {
                if (DDRunDropDownAction.this.fMenu != null) {
                    DDRunDropDownAction.this.fMenu.dispose();
                }
                update();
            }

        });

    }

    protected void addActionToMenu(Menu parent, Action action) {
        ActionContributionItem item = new ActionContributionItem(action);
        item.fill(parent, -1);
    }

    public void run() {
        List stack = ((DDRunResultView) this.fView).getDDRunResultStack();

        if (stack.size() > 1) {
            IDDRunResult ddRunResult = (IDDRunResult) stack.get(1);
            this.fView.display(ddRunResult);
        }
    }
}