/*
 * $Id: GraphDiffVisualizer.java,v 1.25 2005/10/07 19:51:32 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Oct 1, 2004 12:44:13 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer;

import java.util.Iterator;
import java.util.Vector;

import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.logger.DDLogger;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.filter.nodefilter.GraphNodeDifferentFilter;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizGraphEdge;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizGraphElement;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizGraphNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.exceptions.GraphVisualizerException;

/**
 * COMMENTME Comment type GraphDiffVisualizer
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphDiffVisualizer extends AbstractGraphVisualizer {

    // Personal logger for this class
    private final static DDLogger logger = VisualizerPlugin
            .getLogger(GraphDiffVisualizer.class);

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#showFullQualifiedNames()
     */
    boolean showFullQualifiedNames() {
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#showCountForHiddenNodes()
     */
    boolean showCountForHiddenNodes() {
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getGraphFilters()
     */
    Vector getGraphFilters() {
        Vector filters = new Vector();
        filters.add(new GraphNodeDifferentFilter());

        return filters;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getOutputDirectory()
     */
    String getOutputDirectory() {
        return getPluginPreferences().getString(
                IVisualizerPreferenceConstants.GRAPHVIZ_MEMGRAPH_PATH);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getNodeContent(org.deltadebugging.ddstate.memorygraph.IGraphNode,
     *      boolean)
     */
    String getNodeContent(VizGraphNode graphNode, boolean showFullQualifiedNames)
            throws GraphVisualizerException {

        String result = graphNode.getGraphVizName(showFullQualifiedNames);

        if (VisualizerPlugin.DEBUG_GRAPH_DIFF_VISUALIZER_SHOW_FLAGS) {
            result += " " + this.generateFlags(graphNode);
        }

        if (graphNode.isMaskedAs(GraphDecorator.DIFFERENT)) {
            assert graphNode.hasAssociatedElement();

            VizGraphNode assNode = (VizGraphNode) graphNode
                    .getAssociatedElement();

            result += " | " + assNode.getGraphVizName(showFullQualifiedNames);

            if (VisualizerPlugin.DEBUG_GRAPH_DIFF_VISUALIZER_SHOW_FLAGS) {
                result += " " + this.generateFlags(assNode);
            }

            result = result.replaceAll("<", "\\\\<");
            result = result.replaceAll(">", "\\\\>");
        }

        return result;
    }

    private String generateFlags(VizGraphElement vizGraphElement) {
        String result = "";

        IGraphElement graphElment = vizGraphElement.getGraphElement();

        // Passing graph or failing graph flag
        result += graphElment.getRootNode().isPassingGraph() ? "P" : "F";

        // is in the maximum common subgraph
        result += graphElment.hasAssociatedElement() ? "M" : "-";

        // is diff visualizer flag is set
        result += graphElment.isDiffVisualizerFlagSet() ? "D" : "-";

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getEdgeContent(org.deltadebugging.ddstate.memorygraph.IGraphEdge,
     *      boolean)
     */
    String getEdgeContent(VizGraphEdge graphEdge, boolean showFullQualifiedNames) {
        String result = graphEdge.getGraphVizName(showFullQualifiedNames);

        if (VisualizerPlugin.DEBUG_GRAPH_DIFF_VISUALIZER_SHOW_FLAGS) {
            result += " " + this.generateFlags(graphEdge);
        }

        // result += " " + graphEdge.getDifferenceMask();
        // result += " (" + graphEdge.getName() + ")";

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getNodeAttributes(org.deltadebugging.ddstate.memorygraph.IGraphNode)
     */
    Iterator getNodeAttributes(VizGraphNode graphNode) {
        GraphVizAttributeList attributes = graphNode.getGraphVizAttributes();

        if (graphNode.isMaskedAs(GraphDecorator.DIFFERENT)) {
            attributes.addAttribute(GraphVizAttribute.SHAPE_RECORD);
        } else {            
            attributes.addAttribute(GraphVizAttribute.SHAPE_BOX);
        }
        
        if (graphNode.isMaskedAs(GraphDecorator.EQUAL)) {
        	attributes.addAttribute(GraphVizAttribute.STYLE_DEFAULT);
        }

        GraphVizAttribute defaultColor = null;
        GraphVizAttribute fontColor = null;

        if (PRESENTATION) {
            fontColor = GraphVizAttribute.FONTCOLOR_WHITE;
            defaultColor = GraphVizAttribute.FILLCOLOR_BLACK;
        } else {
            fontColor = GraphVizAttribute.FONTCOLOR_BLACK;
            defaultColor = GraphVizAttribute.COLOR_BLACK;
        }

        this.colorGraphElements(graphNode, attributes, defaultColor, fontColor);

        return attributes.iterator();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getEdgeAttributes(org.deltadebugging.ddstate.memorygraph.IGraphEdge)
     */
    Iterator getEdgeAttributes(VizGraphEdge graphEdge) {
        GraphVizAttributeList attributes = new GraphVizAttributeList();

        GraphVizAttribute defaultColor = null;
        GraphVizAttribute fontColor = null;
        if (PRESENTATION) {
            defaultColor = GraphVizAttribute.COLOR_WHITE;
            fontColor = GraphVizAttribute.FONTCOLOR_WHITE;
        } else {
            defaultColor = GraphVizAttribute.COLOR_BLACK;
            fontColor = GraphVizAttribute.FONTCOLOR_BLACK;
        }

        this.colorGraphElements(graphEdge, attributes, defaultColor, fontColor);

        return attributes.iterator();
    }

    private void colorGraphElements(VizGraphElement graphElement,
            GraphVizAttributeList attributes, GraphVizAttribute defaultColor,
            GraphVizAttribute fontColor) {
        attributes.addAttribute(fontColor);

        GraphVizAttribute colorGreen = null;
        GraphVizAttribute colorRed = null;
        GraphVizAttribute colorOrange = null;
        GraphVizAttribute colorYellow = null;

//        if (PRESENTATION) {
//            colorGreen = GraphVizAttribute.COLOR_DARK_GREEN;
//            colorRed = GraphVizAttribute.COLOR_DARK_RED;
//            colorOrange = GraphVizAttribute.COLOR_DARK_ORANGE;
//            colorYellow = GraphVizAttribute.COLOR_DARK_YELLOW;
//        } else {
            colorGreen = GraphVizAttribute.COLOR_GREEN;
            colorRed = GraphVizAttribute.COLOR_RED;
            colorOrange = GraphVizAttribute.COLOR_ORANGE;
            colorYellow = GraphVizAttribute.COLOR_YELLOW;
//        }

        if (graphElement.isMaskedAs(GraphDecorator.EQUAL)) {
            // EQUAL
            attributes.addAttribute(defaultColor);

        } else if (graphElement.isMaskedAs(GraphDecorator.DIFFERENT)) {
            // DIFFERENT
            attributes.addAttribute(colorOrange);

        } else if (graphElement.isMaskedAs(GraphDecorator.ONLY_IN_FIRSTGRAPH)
                || graphElement.isMaskedAs(GraphDecorator.ONLY_IN_SECONDGRAPH)) {
            // ONLY_IN_FIRSTGRAPH || ONLY_IN_GRAPH_TWO
            GraphVizAttribute color = graphElement.getRootNode()
                    .isPassingGraph() ? colorGreen : colorRed;

            attributes.addAttribute(color);

        } else if (graphElement.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
            // NOT_COMPARED
            attributes.addAttribute(colorYellow);

        } else {
            // DEFAULT
            attributes.addAttribute(defaultColor);
        }
    }

    private Vector mergingAssociatedGraphNodes(VizGraphNode firstNode,
            VizGraphNode secondNode) throws GraphVisualizerException {

        assert firstNode.getAssociatedElement() == secondNode;
        assert secondNode.getAssociatedElement() == firstNode;

        Vector result = new Vector();

        // first put all the nodes of the first graph node in the result vector
        result.addAll(firstNode.getSuccessors());

        // look for successors in the second graph that are only available
        // in this
        for (Iterator itSuccessors = secondNode.getSuccessors().iterator(); itSuccessors
                .hasNext();) {

            VizGraphEdge succEdge = (VizGraphEdge) itSuccessors.next();

            if (succEdge.isMaskedAs(GraphDecorator.ONLY_IN_FIRSTGRAPH)
                    || succEdge.isMaskedAs(GraphDecorator.ONLY_IN_SECONDGRAPH)) {

                result.add(succEdge);
                logger.trace(VisualizerPlugin.DEBUG_GRAPH_DIFF_VISUALIZER,
                        "Add edge " + succEdge.getGraphVizName(false)
                                + " to Node "
                                + firstNode.getGraphVizName(false));
            }
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer#getAllSuccessors(org.deltadebugging.ddstate.memorygraph.IGraphNode)
     */
    Iterator getAllSuccessors(VizGraphNode graphNode)
            throws GraphVisualizerException {

        Vector result = new Vector();

        if (graphNode.hasAssociatedElement()) {
            VizGraphNode assNode = (VizGraphNode) graphNode
                    .getAssociatedElement();

            result = this.mergingAssociatedGraphNodes(graphNode, assNode);

        } else {
            result.addAll(graphNode.getSuccessors());
        }

        return result.iterator();
    }
}