/*
 * $Id: GraphVizAttribute.java,v 1.7 2005/11/16 15:20:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 15:18:48 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.attributes;

/**
 * This class is a typesafe enumeration for the all public available graph
 * attributes. Every graph element can specify his graph attributes for the
 * visualization. For this the graph elements have to use the here defined
 * constants.
 * 
 * @see <a href="http://www.graphviz.org/">GraphViz - Graph Drawing Tools </a>
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphVizAttribute {

	String name = null;

	String value = null;

	private GraphVizAttribute(String name, String value) {
		this.name = name;
		this.value = value;
	}

	/**
	 * Return the name of the GraphViz attribute.
	 * 
	 * @return the name of the GraphViz attribute.
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Retrun the value of the GraphViz attribute.
	 * 
	 * @return the value of the GraphViz attribute.
	 */
	public String getValue() {
		return this.value;
	}

	public final static GraphVizAttribute SHAPE_BOX = new GraphVizAttribute(
			"shape", "box");

	public final static GraphVizAttribute SHAPE_PLAINTEXT = new GraphVizAttribute(
			"shape", "plaintext");

	public final static GraphVizAttribute SHAPE_RECORD = new GraphVizAttribute(
			"shape", "record");

	public final static GraphVizAttribute COLOR_RED = new GraphVizAttribute(
			"color", "#FF2B00");

	public final static GraphVizAttribute COLOR_DARKRED = new GraphVizAttribute(
			"color", "red3");
    
    public final static GraphVizAttribute COLOR_PASTELRED = new GraphVizAttribute(
            "color", "#ffc7c1");

	public final static GraphVizAttribute COLOR_GREEN = new GraphVizAttribute(
			"color", "green");

	public final static GraphVizAttribute COLOR_DARK_GREEN = new GraphVizAttribute(
			"color", "green3");

	public final static GraphVizAttribute COLOR_BLUE = new GraphVizAttribute(
			"color", "#004CFF");

	public final static GraphVizAttribute COLOR_DARKBLUE = new GraphVizAttribute(
			"color", "#0000BA");
    
    public final static GraphVizAttribute COLOR_PASTELBLUE = new GraphVizAttribute(
            "color", "#cbcbff");

	public final static GraphVizAttribute COLOR_YELLOW = new GraphVizAttribute(
			"color", "#FFFF66");

	public final static GraphVizAttribute COLOR_DARKYELLOW = new GraphVizAttribute(
			"color", "yellow3");
    
    public final static GraphVizAttribute COLOR_PASTELYELLOW = new GraphVizAttribute(
            "color", "#ffefb9");

	public final static GraphVizAttribute COLOR_WHITE = new GraphVizAttribute(
			"color", "white");

	public final static GraphVizAttribute COLOR_BLACK = new GraphVizAttribute(
			"color", "black");

    public final static GraphVizAttribute COLOR_PASTELORANGE = new GraphVizAttribute(
            "color", "#ffe4b3");
    
	public final static GraphVizAttribute COLOR_ORANGE = new GraphVizAttribute(
			"color", "orange");

	public final static GraphVizAttribute COLOR_DARK_ORANGE = new GraphVizAttribute(
			"color", "orange3");

	public final static GraphVizAttribute FILLCOLOR_WHITE = new GraphVizAttribute(
			"fillcolor", "white");

	public final static GraphVizAttribute FILLCOLOR_BLACK = new GraphVizAttribute(
			"fillcolor", "black");

	public final static GraphVizAttribute FONTCOLOR_WHITE = new GraphVizAttribute(
			"fontcolor", "white");

	public final static GraphVizAttribute FONTCOLOR_BLACK = new GraphVizAttribute(
			"fontcolor", "black");

	public final static GraphVizAttribute STYLE_FILLED = new GraphVizAttribute(
			"style", "filled");

	public final static GraphVizAttribute STYLE_DEFAULT = new GraphVizAttribute(
			"style", "");

	public final static GraphVizAttribute COLOR_DEFAULT_NODE = COLOR_WHITE;

	public final static GraphVizAttribute COLOR_DEFAULT_EDGE = COLOR_BLACK;

}