/*
 * $Id: GraphVizPreferencePage.java,v 1.10 2005/08/12 12:15:36 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.09.2004 16:00:47 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.ui.preferences;

import java.io.File;

import org.deltadebugging.ddstate.visualizer.IVisualizerPreferenceConstants;
import org.deltadebugging.ddstate.visualizer.VisualizerPlugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * COMMENTME Comment type PreferencePage
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphVizPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {

	Combo txtHome;

	Button butEnable;

	Button butFullQualiNames;

	Button butFilterTypes;

	private SelectionListener browseButtonSelectionListener = new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			handleBrowse();
			validatePage();
		}
	};

	/**
	 * Constructor of the preference page
	 * 
	 */
	public GraphVizPreferencePage() {
		this.setDescription("Here you can set the directory for the location "
				+ "of the generated visualizations of the memory graphs. \n "
				+ "You can use dotty, a tool from the GraphViz tool library "
				+ "(see: http:\\\\www.graphviz.org) to show a graphical "
				+ "represantation of the graph. \n");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		Font font = parent.getFont();

		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.marginHeight = layout.marginWidth = 0;
		container.setLayout(layout);
		container.setFont(font);

		// button to enable graph visualization
		new Label(container, SWT.NONE);

		this.butEnable = new Button(container, SWT.CHECK);
		this.butEnable.setText("Enable graph visualization");
		this.butEnable.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		this.butEnable.setFont(font);

		// initialize the group for the preferences
		final Group grpHome = new Group(container, SWT.SHADOW_ETCHED_IN);
		grpHome.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		grpHome.setLayout(new GridLayout(2, false));
		grpHome.setText("");
		grpHome.setFont(font);

		Label lblHome = null;
		Button butBrowse = null;

		lblHome = new Label(grpHome, SWT.NULL);
		lblHome.setText("Location:");
		lblHome.setFont(font);

		this.txtHome = new Combo(grpHome, SWT.NONE);
		this.txtHome.setLayoutData(this
				.getGridData(GridData.FILL_HORIZONTAL, 2));
		this.txtHome.setFont(font);
		this.txtHome.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				validatePage();
			}
			
		});

		butBrowse = new Button(grpHome, SWT.PUSH);

		butBrowse.setText("Browse");
		butBrowse.setLayoutData(this.getGridData(GridData.HORIZONTAL_ALIGN_END,
				2));
		butBrowse.setFont(font);
		butBrowse.addSelectionListener(this.browseButtonSelectionListener);

		this.butFullQualiNames = new Button(grpHome, SWT.CHECK);
		this.butFullQualiNames.setText("Show full qualified Names");
		this.butFullQualiNames.setLayoutData(this.getGridData(
				GridData.HORIZONTAL_ALIGN_BEGINNING, 2));
		this.butFullQualiNames.setFont(font);

		this.butFilterTypes = new Button(grpHome, SWT.CHECK);
		this.butFilterTypes.setText("Filter Types");
		this.butFilterTypes.setLayoutData(this.getGridData(
				GridData.HORIZONTAL_ALIGN_BEGINNING, 2));
		this.butFilterTypes.setFont(font);

		// add selection listener to the enable button
		this.butEnable.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				grpHome.setEnabled(GraphVizPreferencePage.this.butEnable
						.getSelection());
				validatePage();
			}
		});

		// initialize the various fields with the values from the saved
		// preferences
		this.initializeValues();

		// enable or disable the group contents
		grpHome.setEnabled(this.butEnable.getSelection());

		return container;
	}

	/**
	 * Initialize the various fields of this page with the saved preferences.
	 * 
	 */
	private void initializeValues() {
		IPreferenceStore store = getPreferenceStore();

		this.butEnable
				.setSelection(store
						.getBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_ENABLE_VISUALIZATION));
		this.txtHome
				.setText(store
						.getString(IVisualizerPreferenceConstants.GRAPHVIZ_MEMGRAPH_PATH));
		this.butFullQualiNames
				.setSelection(store
						.getBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_SHOW_FULL_QUALIFIED_NAMES));
		this.butFilterTypes
				.setSelection(store
						.getBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_FILTER_TYPES));
	}

	/**
	 * Initialize the various fields of this page with the default values of the
	 * preferences.
	 * 
	 */
	private void initializeDefaults() {
		IPreferenceStore store = getPreferenceStore();

		this.butEnable
				.setSelection(store
						.getDefaultBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_ENABLE_VISUALIZATION));
		this.txtHome
				.setText(store
						.getDefaultString(IVisualizerPreferenceConstants.GRAPHVIZ_MEMGRAPH_PATH));
		this.butFullQualiNames
				.setSelection(store
						.getDefaultBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_SHOW_FULL_QUALIFIED_NAMES));
		this.butFilterTypes
				.setSelection(store
						.getDefaultBoolean(IVisualizerPreferenceConstants.GRAPHVIZ_FILTER_TYPES));
	}

	/**
	 * Store the various field in the preferences referenced to this preference
	 * page.
	 * 
	 */
	private void storeValues() {
		IPreferenceStore store = getPreferenceStore();

		store.setValue(
				IVisualizerPreferenceConstants.GRAPHVIZ_ENABLE_VISUALIZATION,
				this.butEnable.getSelection());
		store.setValue(IVisualizerPreferenceConstants.GRAPHVIZ_MEMGRAPH_PATH,
				this.txtHome.getText());
		store
				.setValue(
						IVisualizerPreferenceConstants.GRAPHVIZ_SHOW_FULL_QUALIFIED_NAMES,
						this.butFullQualiNames.getSelection());
		store.setValue(IVisualizerPreferenceConstants.GRAPHVIZ_FILTER_TYPES,
				this.butFilterTypes.getSelection());
	}

	/**
	 * Returns preference store that belongs to the our plugin.
	 * 
	 * @return the preference store for this plugin
	 */
	protected IPreferenceStore doGetPreferenceStore() {
		return VisualizerPlugin.getDefault().getPreferenceStore();
	}

	void validatePage() {
		this.setErrorMessage(null);

		if (this.butEnable.getSelection()) {

			String location = this.txtHome.getText().trim();

			if (location.equals("")) {
				this
						.setErrorMessage("Specify a location for the memory graphs");
				this.setValid(false);
				return;
			}

			File ioLoc = new File(location);

			if (!ioLoc.exists()) {
				this
						.setErrorMessage("Specify a valid location. The given location does not exists");
				this.setValid(false);
				return;
			}

			if (!ioLoc.isDirectory()) {
				this
						.setErrorMessage("Specify a valid location. The given location is not a directory");
				this.setValid(false);
				return;
			}
		}
		
		this.setValid(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		this.initializeDefaults();

		super.performDefaults();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferencePage#isValid()
	 */
	public boolean isValid() {
		return this.getErrorMessage() == null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		this.storeValues();

		return super.performOk();
	}

	private GridData getGridData(int style, int horizontalSpan) {
		return this.getGridData(style, horizontalSpan, SWT.DEFAULT);
	}

	private GridData getGridData(int style, int horizontalSpan, int widthHint) {
		GridData result = new GridData(style);
		result.horizontalSpan = horizontalSpan;
		result.widthHint = widthHint;
		return result;
	}

	protected void handleBrowse() {
		DirectoryDialog dialog = new DirectoryDialog(getShell());
		if (this.txtHome.getText().length() > 0)
			dialog.setFilterPath(this.txtHome.getText());

		String newPath = dialog.open();
		if (newPath != null) {
			if (this.txtHome.indexOf(newPath) == -1)
				this.txtHome.add(newPath, 0);
			this.txtHome.setText(newPath);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		// do nothing
	}

}