/*
 * $Id: VizGraphEdge.java,v 1.3 2005/11/17 12:32:15 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 10:41:03 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;

/**
 * The decorator class for the <code>IGraphEdge</code>.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.IGraphEdge
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class VizGraphEdge extends VizGraphElement {

    protected VizGraphEdge(IGraphEdge graphEdge, VizRootNode rootNode) {
        super(graphEdge, rootNode);
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizName(boolean)
     */
    public String getGraphVizName(boolean fullQualified) {
        return ((IGraphEdge) this.getGraphElement()).getName();
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizAttributes()
     */
    public GraphVizAttributeList getGraphVizAttributes() {
        GraphVizAttributeList attrList = new GraphVizAttributeList();

        if (this.hasAssociatedElement())
            attrList.addAttribute(GraphVizAttribute.COLOR_RED);
        else
            attrList.addAttribute(GraphVizAttribute.COLOR_DEFAULT_EDGE);

        return attrList;
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphEdge#getTargetNode()
     */
    public VizGraphNode getTargetNode() {
        return VizGraphElementFactory.createGraphNode(((IGraphEdge) this.getGraphElement())
                .getTargetNode(), this.rootNode);
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphEdge#getSourceNode()
     */
    public VizGraphNode getSourceNode() {
        return VizGraphElementFactory.createGraphNode(((IGraphEdge) this.getGraphElement())
                .getSourceNode(), this.rootNode);
    }
}

