/*
 * $Id: VizGraphElement.java,v 1.4 2005/11/24 12:09:11 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 09:54:46 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;
import org.deltadebugging.ddstate.visualizer.vizgraph.exceptions.GraphVisualizerException;

/**
 * The decorator class for the <code>IGraphElement</code>.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class VizGraphElement implements IVizGraphElement {

    private IGraphElement decoratorObject = null;

    /**
     * The anchor node for the viz graph. It have to used to register all new
     * greated viz graph elements
     */
    protected VizRootNode rootNode = null;

    public VizGraphElement(IGraphElement decoratorObject, VizRootNode rootNode) {
        this.decoratorObject = decoratorObject;
        this.rootNode = rootNode;
    }

    public VizRootNode getRootNode() {
        return new VizRootNode(this.decoratorObject.getRootNode());
    }

    /*
     * @see org.deltadebugging.ddstate.memorygraph.comperator.IGraphElementComparable#isDiffVisualizerFlagSet()
     */
    public boolean isDiffVisualizerFlagSet() {
        return this.decoratorObject.isDiffVisualizerFlagSet();
    }

    /**
     * Return the associated element of this graph element, if one exists, else
     * null.
     * 
     * @return the associated element of this graph element.
     */
    public VizGraphElement getAssociatedElement() {
        VizGraphElement result = null;

        if (this.decoratorObject.hasAssociatedElement()) {
            IGraphElement assElement = this.decoratorObject
                    .getAssociatedElement();

            if (assElement instanceof IGraphEdge) {
                result = VizGraphElementFactory.createGraphEdge((IGraphEdge) assElement,
                        this.rootNode);
            } else if (assElement instanceof IGraphNode) {
                result = VizGraphElementFactory.createGraphNode((IGraphNode) assElement,
                        this.rootNode);
            } else {
                throw new IllegalArgumentException("Graph elements of type "
                        + assElement.getClass().getName()
                        + " are not supported.");
            }
        }

        return result;
    }

    /*
     * @see org.deltadebugging.ddstate.memorygraph.comperator.mcs.IMCSComparable#hasAssociatedElement()
     */
    public boolean hasAssociatedElement() {
        return this.decoratorObject.hasAssociatedElement();
    }

    /*
     * @see org.deltadebugging.ddstate.memorygraph.comperator.delta.IDeltaComparable#getDifferenceMask()
     */
    public int getDifferenceMask() {
        return this.decoratorObject.getDifferenceMask();
    }

    /*
     * @see org.deltadebugging.ddstate.memorygraph.comperator.delta.IDeltaComparable#isMaskedAs(byte)
     */
    public boolean isMaskedAs(byte mask) {
        return this.decoratorObject.isMaskedAs(mask);
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphElement()
     */
    public IGraphElement getGraphElement() {
        return this.decoratorObject;
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizName(boolean)
     */
    public abstract String getGraphVizName(boolean fullQualified)
            throws GraphVisualizerException;

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizAttributes()
     */
    public abstract GraphVizAttributeList getGraphVizAttributes()
            throws GraphVisualizerException;
}

