/*
 * $Id: VizGraphNode.java,v 1.6 2006/01/24 19:48:32 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 09:51:40 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph;

import java.util.Iterator;
import java.util.Vector;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.IGraphNode;
import org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;

/**
 * The decorator class for the <code>IGraphNode</code>.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.IGraphNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class VizGraphNode extends VizGraphElement {

    /**
     * Don't use the constructor to create a new VizGraphNode. See for the
     * static factory methods instead. Only subclasses of the VizGraphNode have
     * to use this constructor.
     * 
     * @param graphNode
     */
    protected VizGraphNode(IGraphNode graphNode, VizRootNode rootNode) {
        super(graphNode, rootNode);
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizAttributes()
     */
    public GraphVizAttributeList getGraphVizAttributes() {

        GraphVizAttributeList attrList = new GraphVizAttributeList();
        attrList.addAttribute(GraphVizAttribute.COLOR_DEFAULT_NODE);
        attrList.addAttribute(GraphVizAttribute.STYLE_FILLED);

        if (AbstractGraphVisualizer.THESIS) {
            attrList.addAttribute(GraphVizAttribute.SHAPE_BOX);
        }
        
        if (this.getGraphElement().hasAssociatedElement())
            attrList.addAttribute(GraphVizAttribute.SHAPE_BOX);

        return attrList;
    }

    public Vector getSuccessors() {
        Vector result = new Vector();

        Iterator itSuccessors = ((IGraphNode) this.getGraphElement())
                .getChildren().iterator();
        while (itSuccessors.hasNext()) {
            IGraphEdge graphEdge = (IGraphEdge) itSuccessors.next();
            result.add(VizGraphElementFactory.createGraphEdge(graphEdge,
                    this.rootNode));
        }

        return result;
    }

    public Iterator getParents() {
        Vector result = new Vector();

        Iterator itPredecessors = ((IGraphNode) this.getGraphElement())
                .getParents().iterator();
        while (itPredecessors.hasNext()) {
            IGraphEdge graphEdge = (IGraphEdge) itPredecessors.next();
            result.add(VizGraphElementFactory.createGraphEdge(graphEdge,
                    this.rootNode));
        }

        return result.iterator();
    }
}

