/*
 * $Id: VizPrimitiveValueNode.java,v 1.1 2005/11/14 12:58:25 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 16.12.2004 13:37:55 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.visualizer.vizgraph.value;

import org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode;
import org.deltadebugging.ddstate.visualizer.AbstractGraphVisualizer;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttribute;
import org.deltadebugging.ddstate.visualizer.attributes.GraphVizAttributeList;
import org.deltadebugging.ddstate.visualizer.vizgraph.VizRootNode;
import org.deltadebugging.ddstate.visualizer.vizgraph.exceptions.GraphVisualizerException;
import org.eclipse.jdt.debug.core.IJavaPrimitiveValue;

/**
 * The decorator class for the <code>IPrimitiveValueNode</code>.
 * 
 * @see org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class VizPrimitiveValueNode extends VizValueNode {

    private static final String CHAR = "char";

    public VizPrimitiveValueNode(IPrimitiveValueNode graphNode,
            VizRootNode rootNode) {
        super(graphNode, rootNode);
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizAttributes()
     */
    public GraphVizAttributeList getGraphVizAttributes() {
        GraphVizAttributeList attrList = super.getGraphVizAttributes();

        if (AbstractGraphVisualizer.THESIS) {
            attrList.addAttribute(GraphVizAttribute.COLOR_PASTELBLUE);
            attrList.addAttribute(GraphVizAttribute.FONTCOLOR_BLACK);
        } else {
            attrList.addAttribute(GraphVizAttribute.COLOR_DARKBLUE);
            attrList.addAttribute(GraphVizAttribute.FONTCOLOR_WHITE);
        }

        return attrList;
    }

    /*
     * @see org.deltadebugging.ddstate.visualizer.vizgraph.IVizGraphElement#getGraphVizName(boolean)
     */
    public String getGraphVizName(boolean fullQualified)
            throws GraphVisualizerException {

        String result = null;

        result = this.getReferenceTypeName() + "\\n";

        IJavaPrimitiveValue value = (IJavaPrimitiveValue) this.getValue();

        if (this.getReferenceTypeName().equals(CHAR)) {
            char c = value.getCharValue();
            if (Character.isLetterOrDigit(c)) {
                result += "'" + c + "'";
            } else if (Character.isSpaceChar(c)) {
                result += "'  '";
            } else {                
                result += Character.getNumericValue(c);
            }
        } else {
            result += this.getValue().toString();
        }

        return result;
    }

}
