/*
 * $Id: IMCSComparable.java,v 1.3 2005/08/29 07:36:51 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 14.12.2004 13:56:38 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.comparator.mcs;

import org.deltadebugging.ddstate.memorygraph.IGraphElement;

/**
 * Two graphs which want to compared with the MCSComparator have to implement
 * this interface.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IMCSComparable {

	/**
	 * Reset the associated element.
	 */
	public void resetAssociatedElement();

	/**
	 * True if the graph element has an associated graph element in a second
	 * graph, else false.
	 * 
	 * @return true if the graph element has an associated graph element, else
	 *         false.
	 */
	public boolean hasAssociatedElement();

	/**
	 * Return the associated graph element if one exists in the second graph,
	 * else <code>null</code>.
	 * 
	 * @return the associated graph element if one exists, else <cod>null
	 *         </code>.
	 */
	public IGraphElement getAssociatedElement();

	/**
	 * Set the associated graph element of this graph element to the given graph
	 * element.
	 * 
	 * @param graphElement
	 *            associated graph element to set.
	 */
	public void setAssociatedElement(IGraphElement graphElement);

	/**
	 * Proof if this graph element is associated with the given graph element.
	 * For this all the following conditions must hold:
	 * <ul>
	 * <li><code>this.hasAssociatedElement()</code></li>
	 * <li><code>graphElement.hasAsscociatedElement()</code></li>
	 * <li><code>this.getAssociatedElement() == graphElement</code></li>
	 * <li><code>graphElement.getAssociatedElement() == this</code></li>
	 * </ul>
	 * 
	 * @param graphElement
	 * 
	 * @return true if this is associated with the given graphElement, else
	 *         false
	 */
	public boolean isAssociatedWith(IGraphElement graphElement);

}
