/*
 * $Id: IGraphNode.java,v 1.17 2006/01/24 19:45:32 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 31.08.2004 14:10:29 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph;

import java.util.Collection;
import java.util.SortedSet;

import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.deltadebugging.ddstate.memorygraph.type.IGlobalVariableEdge;
import org.deltadebugging.ddstate.memorygraph.value.IArrayElementEdge;
import org.deltadebugging.ddstate.memorygraph.value.IObjectFieldEdge;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * COMMENTME Comment type IGraphNode
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public interface IGraphNode extends IGraphElement {

    /**
     * Looks for all successors in this graph node and create recursivly all
     * successors from it until the entire graph is build.
     * 
     * @param the
     *            progress monitor.
     * 
     * @throws MemoryGraphExtractionException
     *             if something went wrong on the exctraction of the memory
     *             graph.
     */
    public void unfoldSubgraph(IProgressMonitor monitor)
            throws MemoryGraphExtractionException;

    /**
     * @return Returns the children.
     */
    public SortedSet getChildren();

    /**
     * Returns true if this node represents a local variable. This means that
     * the variable is direct accessible from a stackframe.
     * 
     * @return true if this node represents a local variable, otherwise false.
     */
    public boolean isLocalVariable();

    /**
     * Returns all predecessors to this variable from type ILocalVariableEdge.
     * This predecessors represents all stackframes that direct access the
     * variable of this node.
     * 
     * @return all stackframe predecessors of the variable of this node.
     */
    public ILocalVariableEdge[] getLocalVariablePrecessors();

    /**
     * Returns true if this node represents a global variable. This means that
     * the variable is direct accessible from a reference type.
     * 
     * @return true if this node represents a global variable, otherwise false.
     */
    public boolean isGlobalVariable();

    /**
     * Returns all predecessors to this variable from type IGlobalVariableEdge.
     * This predecessors represents all reference types that direct access the
     * variable of this node.
     * 
     * @return all reference type predecessors of the variable of this node.
     */
    public IGlobalVariableEdge[] getGlobalVariablePredecessors();

    /**
     * Returns true if this node represents a variable referenced by an array.
     * 
     * @return true if this node represents an array element, otherwise false.
     */
    public boolean isArrayElementVariable();

    /**
     * Returns all predecessors to this array element variable from an array.
     * This predecessors represents all arrays that direct acces this variable.
     * 
     * @return all array reference predecessors of the variable of this node.
     */
    public IArrayElementEdge[] getArrayElementPredecessors();

    /**
     * Returns true if this node represents a varialbe referenced by an object
     * reference.
     * 
     * @return true if this node represents a varialbe referenced by an object
     *         reference.
     */
    public boolean isObjectReferenceFieldVariable();

    /**
     * Returns all predecessors to this object reference field variable from an
     * object reference. This predecessors represents all object references
     * pointing to this variable in one of more their fields.
     * 
     * @return all object reference predecessors of the variable of this node.
     */
    public IObjectFieldEdge[] getObjectReferenceFieldPredecessors();

    // COMMENTME addParents
    public void addParent(IGraphEdge graphEdge);

    // COMMENTME getParents
    public Collection getParents();

}