/*
 * $Id: CreateDelta.java,v 1.9 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 29.07.2005 18:46:54 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.create;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.delta.ICreateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.MemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.location.CreateObjectDeltaLocation;
import org.deltadebugging.ddstate.memorygraph.delta.location.IDeltaLocation;
import org.deltadebugging.ddstate.memorygraph.delta.location.PathLocation;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;

/**
 * The create delta creates a new value in the memory. This can either be an
 * object or a primitive value.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class CreateDelta extends MemoryGraphDelta implements
		ICreateDelta {

	/** The graph node of the value that should create */
	private final IValueNode fValueToCreate;

	private final PathLocation fValueToCreateLocation;

	public CreateDelta(IValueNode valueToCreate) {
		super();

		this.fValueToCreate = valueToCreate;
		this.fValueToCreateLocation = new PathLocation(this.fValueToCreate,
				false);
	}

	public CreateDelta(IDeltaLocation deltaLocation,
			IValueNode valueToCreate) {
		super(deltaLocation);

		this.fValueToCreate = valueToCreate;
		this.fValueToCreateLocation = new PathLocation(this.fValueToCreate,
				false);
	}

	public CreateDelta(CreateObjectDelta createDelta, IGraphEdge graphEdge,
			IValueNode valueToCreate) {
		super(new CreateObjectDeltaLocation(createDelta, graphEdge));

		this.fValueToCreate = valueToCreate;
		this.fValueToCreateLocation = new PathLocation(this.fValueToCreate,
				false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.ICreateDelta#getValueToCreate()
	 */
	public IValueNode getValueToCreate() {
		return this.fValueToCreate;
	}

	/**
	 * Returns the location in the memory graph to the value that will created
	 * by this delta.
	 * 
	 * @return the location in the memory graph to the value that will created
	 *         by this delta.
	 */
	public PathLocation getLocationToValueToCreate() {
		return this.fValueToCreateLocation;		
	}

}
