/*
 * $Id: CreatePrimitiveValueDelta.java,v 1.8 2005/11/22 14:23:15 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 29.07.2005 18:40:39 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.delta.create;

import org.deltadebugging.ddstate.launcher.client.ITestRunner;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;
import org.deltadebugging.ddstate.memorygraph.delta.ICreateDelta;
import org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta;
import org.deltadebugging.ddstate.memorygraph.delta.location.PathLocation;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdateArraySizeDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaExecutionException;
import org.deltadebugging.ddstate.memorygraph.util.MemoryGraphUtil;
import org.deltadebugging.ddstate.memorygraph.util.Path;
import org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode;
import org.deltadebugging.ddstate.memorygraph.value.IValueNode;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaThread;
import org.eclipse.jdt.debug.core.IJavaVariable;

/**
 * This delta create a primitive value if it will applied.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class CreatePrimitiveValueDelta extends CreateDelta implements
		ICreateDelta {

	public CreatePrimitiveValueDelta(UpdateArraySizeDelta updateDelta,
			IGraphEdge edgeToPrimtive) {
		super((IValueNode) edgeToPrimtive.getTargetNode());

		Path pathToPrimitive = MemoryGraphUtil.searchPathToRoot(updateDelta
				.getNewValue(), true);
		pathToPrimitive.addLastGraphEdge(edgeToPrimtive);
		this.setDeltaLocation(new PathLocation(pathToPrimitive));

		this.addDependeny(updateDelta);
	}

	public CreatePrimitiveValueDelta(CreateObjectDelta createDelta,
			IGraphEdge graphEdge, IPrimitiveValueNode newValue) {
		super(createDelta, graphEdge, newValue);

		this.addDependeny(createDelta);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#apply(org.eclipse.jdt.debug.core.IJavaThread,
	 *      org.deltadebugging.ddstate.launcher.client.ITestRunner)
	 */
	public void apply(IJavaThread thread, ITestRunner testRunner)
			throws DeltaExecutionException {
		try {
			IJavaVariable variable = this.getLocation().getVariableToLocation(
					thread, testRunner);
			variable.setValue(((IPrimitiveValueNode) this
					.getValueToCreate()).getStringValue());

		} catch (DebugException e) {
			throw new DeltaExecutionException("Cannot apply delta", e);
		} catch (Throwable e) {
			throw new DeltaExecutionException("Cannot apply delta", e);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta#ctsEqual(org.deltadebugging.ddstate.memorygraph.delta.IMemoryGraphDelta)
	 */
	public boolean ctsEqual(IMemoryGraphDelta delta) {

		if (!(delta instanceof CreatePrimitiveValueDelta)) {
			return false;
		}

		CreatePrimitiveValueDelta createObjDelta = (CreatePrimitiveValueDelta) delta;

		return this.getLocationToValueToCreate().equals(
				createObjDelta.getLocationToValueToCreate());
	}

}