/*
 * $Id: TypeNode.java,v 1.2 2005/11/17 12:32:28 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.type;

import org.deltadebugging.ddstate.memorygraph.GraphNode;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphDebugException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaType;

/**
 * A graph node that maps to the fType of a value on a Java debug target.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public abstract class TypeNode extends GraphNode implements ITypeNode {

	private IJavaType fType = null;

	private String fTypeName = null;

	/**
	 * @param rootNode
	 * @param name
	 * @throws MemoryGraphExtractionException
	 */
	public TypeNode(IRootNode rootNode, IJavaType type)
			throws MemoryGraphExtractionException {
		super(rootNode);
		this.fType = type;

		// We have to read the reference fType name from the value. This is
		// the fType name on run time. If you read the reference fType name
		// from the fVariable you get only the fType of the compile time.
		try {
			this.fTypeName = this.fType.getName();
		} catch (DebugException e) {
			throw new MemoryGraphExtractionException(
					"Cannot read the fType name of this reference fType", e);
		}
	}

	/**
	 * @return the tpe of this graph node.
	 */
	public IJavaType getType() {
		return this.fType;
	}

	/**
	 * @return the type name of this graph node.
	 * 
	 * @throws MemoryGraphDebugException
	 *             if something went wrong on retrieving the type name from the
	 *             vm.
	 */
	public String getTypeName() {
		return this.fTypeName;
	}

	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof TypeNode)) {
			return false;
		}

		boolean result = this.getTypeName().equals(
				((TypeNode) element).getTypeName());

		return result;
	}

}