/*
 * $Id: PathComparator.java,v 1.1 2005/08/11 21:15:16 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 11, 2005 8:01:23 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.util;

import java.util.Comparator;

import org.deltadebugging.ddstate.comparator.GraphEdgeComparator;
import org.deltadebugging.ddstate.memorygraph.IGraphEdge;

/**
 * The path comparator compares two paths. Two path are equal if they have equal
 * size and every path element is equal in terms of the GraphEdgeComparator.
 * 
 * @see org.deltadebugging.ddstate.comparator.GraphEdgeComparator
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class PathComparator implements Comparator {

	private static PathComparator instance = null;

	private final static GraphEdgeComparator EDGE_COMPARATOR = GraphEdgeComparator
			.getInstance();

	/**
	 * Designed as singleton.
	 */
	private PathComparator() {
		// empty constructor
	}

	public final static PathComparator getInstance() {
		if (instance == null) {
			instance = new PathComparator();
		}

		return instance;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	public int compare(Object firstObj, Object secondObj) {

		if (!(firstObj instanceof Path && secondObj instanceof Path))
			throw new ClassCastException("Given objects must be from type "
					+ Path.class.getName());

		Path firstPath = (Path) firstObj;
		Path secondPath = (Path) secondObj;

		int result = 0;

		int firstPathSize = firstPath.size();
		int secondPathSize = secondPath.size();

		// first sort after size
		if (firstPathSize != secondPathSize) {
			result = firstPathSize - secondPathSize;

		} else {

			for (int i = 0; i < firstPathSize && result == 0; i++) {
				IGraphEdge firstPathEdge = firstPath.get(i);
				IGraphEdge secondPathEdge = secondPath.get(i);

				result = EDGE_COMPARATOR.compare(firstPathEdge, secondPathEdge);
			}
		}

		return result;
	}
}
