/*
 * $Id: ArrayElementEdge.java,v 1.5 2005/11/28 15:28:44 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.08.2005 13:46:46 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddstate.memorygraph.GraphEdge;

/**
 * This edge joined an array reference with one of his element. In addition to
 * ordinary graph edges this edge provides a method to get the index of the
 * array element. Also a method for retrieving the array length is implemented
 * too.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class ArrayElementEdge extends GraphEdge implements IArrayElementEdge {

    private final int fArrayIndex;

    public ArrayElementEdge(IArrayNode sourceNode, IValueNode targetNode,
            int arrayIndex) {
        super(sourceNode, targetNode, "[" + arrayIndex + "]");
        this.fArrayIndex = arrayIndex;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IArrayElementEdge#getArrayIndex()
     */
    public int getArrayIndex() {
        return this.fArrayIndex;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deltadebugging.ddstate.memorygraph.value.IArrayElementEdge#getArrayLenght()
     */
    public int getArrayLenght() {
        return ((IArrayNode) this.getSourceNode()).getLength();
    }

}
