/*
 * $Id: ClassObjectNode.java,v 1.4 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Sep 28, 2005 5:46:18 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaClassObject;

/**
 * Represents a class reference in the memory graph.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class ClassObjectNode extends ObjectNode implements IClassObjectNode {

	private final String fInstanceName;

	public ClassObjectNode(IRootNode rootNode, IJavaClassObject value)
			throws MemoryGraphExtractionException {
		super(rootNode, value);

		try {
			this.fInstanceName = value.getInstanceType().getName();
		} catch (DebugException e) {
			throw new MemoryGraphExtractionException(
					"Cannot retrieve instance type of class");
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createSuccessors()
	 */
	protected void createChildren() throws MemoryGraphExtractionException {
		// we handle class nodes atomar
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#computeDeltaSet(org.deltadebugging.ddstate.memorygraph.IGraphElement)
	 */
	public DeltaSet computeDeltaSet(IGraphElement element)
			throws DeltaComputationException {

		if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
			return new DeltaSet();
		}

		assert element instanceof IClassObjectNode;
		IClassObjectNode classRef = (IClassObjectNode) element;
		assert classRef.getInstanceType().equals(this.getInstanceType());

		GraphDecorator.decorateGraphElementsAsEqual(this, element);

		return new DeltaSet();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#graphEquals(org.deltadebugging.ddstate.memorygraph.IGraphElement)
	 */
	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof IClassObjectNode)) {
			return false;
		}

		IClassObjectNode objRef = ((IClassObjectNode) element);

		boolean result = super.graphEquals(element)
				&& this.getInstanceType().equals(objRef.getInstanceType());

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.value.IClassObjectNode#getInstanceType()
	 */
	public String getInstanceType() {
		return this.fInstanceName;
	}

}
