/*
 * $Id: IObjectFieldEdge.java,v 1.1 2005/11/14 12:58:27 lehmann Exp $
 * 
 * Copyright (C) 2005 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on Aug 26, 2005 1:37:03 PM by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddstate.memorygraph.IGraphEdge;

/**
 * This edge joins a field node (IValueNode) with the class object
 * reference (IClassObjectReferenceNode) that contains the field.
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann</a>
 */
public interface IObjectFieldEdge extends IGraphEdge {

    /**
     * Returns the full qualified type name of the type which declares the field
     * referenced by this edge.
     * 
     * @return the full qualified type name of the type which declares the field
     *         referenced by this edge.
     */
    public String getDeclaringTypeName();

    /**
     * Returns the JNI-style signature for the declaring type.
     * 
     * @return the JNI-style signature for the declaring type.
     */
    public String getDeclaringTypeSignature();

    /**
     * Returns the name of the field in the type name that declared the field.
     * 
     * @return the name of the field in the type name that declared the field.
     */
    public String getFieldName();

    /**
     * Returns true if the declared field is a field of the super class.
     * 
     * @return true if the declared field is a field of the super class.
     */
    public boolean isSuperclassField();

}
