/*
 * $Id: PrimitiveValueNode.java,v 1.4 2006/01/03 21:02:54 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddcore.DeltaSet;
import org.deltadebugging.ddstate.comparator.delta.GraphDecorator;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.delta.update.UpdatePrimitiveValueDelta;
import org.deltadebugging.ddstate.memorygraph.exceptions.DeltaComputationException;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.jdt.debug.core.IJavaPrimitiveValue;

/**
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 * 
 * COMMENTME Comment type ***
 */
public class PrimitiveValueNode extends ValueNode implements
		IPrimitiveValueNode {

	// COMMENTME
	public PrimitiveValueNode(IRootNode rootNode, IJavaPrimitiveValue value)
			throws MemoryGraphExtractionException {
		super(rootNode, value);
	}

	/*
	 * @see org.deltadebugging.ddstate.memorygraph.GraphNode#createSuccessors()
	 */
	protected void createChildren() {
		// primitive values have not successors
		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.value.IPrimitiveValueNode#getStringValue()
	 */
	public String getStringValue() {
		return this.getValue().toString();
	}

	public DeltaSet computeDeltaSet(IGraphElement element)
			throws DeltaComputationException {

		if (!this.isMaskedAs(GraphDecorator.NOT_COMPARED)) {
			return new DeltaSet();
		}

		DeltaSet result = new DeltaSet();

		assert element instanceof IPrimitiveValueNode;
		IPrimitiveValueNode primValue = (IPrimitiveValueNode) element;
		assert this.getReferenceTypeName().equals(
				primValue.getReferenceTypeName());

		if (!this.getStringValue().equals(primValue.getStringValue())) {
			// mask as changed
			GraphDecorator.decorateGraphElementsAsDifferent(this, primValue);

			result.add(new UpdatePrimitiveValueDelta(this, primValue));

		} else {
			// mask as equal
			GraphDecorator.decorateGraphElementsAsEqual(this, element);
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deltadebugging.ddstate.memorygraph.IGraphElement#graphEquals(org.deltadebugging.ddstate.memorygraph.IGraphElement)
	 */
	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof IPrimitiveValueNode)) {
			return false;
		}

		IPrimitiveValueNode primitiveObj = (IPrimitiveValueNode) element;

		boolean result = false;

		result = super.graphEquals(element)
				&& this.getStringValue().equals(primitiveObj.getStringValue());

		return result;
	}

}