/*
 * $Id: ValueNode.java,v 1.2 2005/11/17 12:32:28 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 26.08.2004 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.memorygraph.value;

import org.deltadebugging.ddstate.memorygraph.GraphNode;
import org.deltadebugging.ddstate.memorygraph.IGraphElement;
import org.deltadebugging.ddstate.memorygraph.IRootNode;
import org.deltadebugging.ddstate.memorygraph.exceptions.MemoryGraphExtractionException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.jdt.debug.core.IJavaValue;

/**
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 * 
 * COMMENTME Comment type ***
 */
public abstract class ValueNode extends GraphNode implements IValueNode {

	private String fReferenceTypeName = null;

	private IJavaValue fValue = null;

	// COMMENTME ValueNode
	public ValueNode(IRootNode rootNode, IJavaValue value)
			throws MemoryGraphExtractionException {
		super(rootNode);

		this.fValue = value;

		// We have to read the reference type name from the value. This is
		// the type name on run time. If you read the reference type name
		// from the fVariable you get only the type of the compile time.
		try {
			this.fReferenceTypeName = value.getReferenceTypeName();
		} catch (DebugException e) {
			throw new MemoryGraphExtractionException(
					"Cannot read the reference TypeNode Name of the value of the given variable",
					e);
		}
	}

	/**
	 * Returns a description of the type of data the value of this Variable
	 * contains or references. Note that the declared type of a fVariable and
	 * the concrete type of its value are not neccessarily the same.
	 * 
	 * @return the name of the concrete type
	 */
	public String getReferenceTypeName() {
		return this.fReferenceTypeName;
	}

	public String getShortReferenceTypeName() {
		String result = this.fReferenceTypeName;

		if (result != null) {
			int lastIndex = result.lastIndexOf('.') + 1;
			result = result.substring(lastIndex, result.length());
		}

		return result;
	}

	// COMMENTME getValue
	public IJavaValue getValue() {
		return this.fValue;
	}

	public String toString() {
		return super.toString() + " (" + this.fReferenceTypeName + ")";
	}

	public boolean graphEquals(IGraphElement element) {
		if (!(element instanceof ValueNode)) {
			return false;
		}

		return this.getReferenceTypeName().equals(
				((ValueNode) element).getReferenceTypeName());
	}
}