/*
 * $Id: GraphExtractorPreferencePage.java,v 1.6 2005/10/11 19:17:10 lehmann Exp $
 * 
 * Copyright (C) 2004 Karsten Lehmann
 * 
 * Software Engineering Chair, Saarland University
 * Prof. Dr. Andreas Zeller
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Created on 03.09.2004 16:00:47 by
 * @author Karsten Lehmann 
 * Mail to: lehmann@cs.uni-sb.de or KarstenLehmann@web.de
 */
package org.deltadebugging.ddstate.ui.preferences;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import org.deltadebugging.ddstate.DDStatePlugin;
import org.deltadebugging.ddstate.IDDStatePreferenceConstants;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.internal.debug.ui.ExceptionHandler;
import org.eclipse.jdt.internal.debug.ui.Filter;
import org.eclipse.jdt.internal.debug.ui.FilterLabelProvider;
import org.eclipse.jdt.internal.debug.ui.FilterViewerSorter;
import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
import org.eclipse.jdt.internal.debug.ui.JavaDebugOptionsManager;
import org.eclipse.jdt.ui.IJavaElementSearchConstants;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.dialogs.SelectionDialog;

/**
 * COMMENTME Comment type DDStatePreferencePage
 * 
 * @author <a href="mailto:KarstenLehmann@web.de">Karsten Lehmann </a>
 */
public class GraphExtractorPreferencePage extends PreferencePage implements
        IWorkbenchPreferencePage {

    // Extraction filter widgets
    private CheckboxTableViewer fTabViewFilter;
    private Table fTabFilter;
    private Button fButAddPackage;
    private Button fButAddType;
    Button fButRemoveFilter;
    private Button fButAddFilter;
    private Button fButEnableAll;
    private Button fButDisableAll;
    private TableItem fTabItemNew;
    private Label fLblTable;

    private Button fEnableAllButton;
    private Button fDisableAllButton;

    FilterContentProvider fFilterContentProvider;

    /**
     * Constractor of the preference page
     *  
     */
    public GraphExtractorPreferencePage() {
        this.setDescription("Here you can define all packages and types "
                + "which the Graph-Extractor should ignore, "
                + "if it extracts the global (static) variables.\n");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
     */
    protected Control createContents(Composite parent) {

        // initialize the various fields with the values from the saved
        // preferences
        this.initializeValues();

        //The main composite
        Composite composite = new Composite(parent, SWT.NULL);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        composite.setLayout(layout);
        GridData data = new GridData();
        data.verticalAlignment = GridData.FILL;
        data.horizontalAlignment = GridData.FILL;
        composite.setLayoutData(data);
        composite.setFont(parent.getFont());

        this.createExtractionFilterPreferences(composite);

        return composite;
    }

    /**
     * Initialize the various fields of this page with the saved preferences.
     *  
     */
    private void initializeValues() {
        // IPreferenceStore store = getPreferenceStore();

        // load the preferences
    }

    /**
     * Initialize the various fields of this page with the default values of the
     * preferences.
     *  
     */
    private void initializeDefaults() {
        // initialize the default values
        this.fFilterContentProvider.setDefaults();
    }

    /**
     * Store the various field in the preferences referenced to this preference
     * page.
     *  
     */
    private void storeValues() {
        // store the preferences
        this.fFilterContentProvider.saveFilters();
    }

    /**
     * Returns preference store that belongs to the our plugin.
     * 
     * @return the preference store for this plugin
     */
    protected IPreferenceStore doGetPreferenceStore() {
        return DDStatePlugin.getDefault().getPreferenceStore();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        initializeDefaults();
        super.performDefaults();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.preference.IPreferencePage#performOk()
     */
    public boolean performOk() {
        storeValues();
        return super.performOk();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init(IWorkbench workbench) {
    		// do nothing
    }

    private void createExtractionFilterButtons(Composite container) {
        Font font = container.getFont();
        initializeDialogUnits(container);
        // button container
        Composite buttonContainer = new Composite(container, SWT.NONE);
        GridData gd = new GridData(GridData.FILL_VERTICAL);
        buttonContainer.setLayoutData(gd);
        GridLayout buttonLayout = new GridLayout();
        buttonLayout.numColumns = 1;
        buttonLayout.marginHeight = 0;
        buttonLayout.marginWidth = 0;
        buttonContainer.setLayout(buttonLayout);

        // Add filter button
        this.fButAddFilter = new Button(buttonContainer, SWT.PUSH);
        this.fButAddFilter.setText("Add filter");
        this.fButAddFilter.setToolTipText("Add filter");
        setButtonLayoutData(this.fButAddFilter);
        this.fButAddFilter.setFont(font);
        this.fButAddFilter.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                addFilter();
            }
        });

        // Add type button
        this.fButAddType = new Button(buttonContainer, SWT.PUSH);
        this.fButAddType.setText("Add type");
        this.fButAddType.setToolTipText("Add type");
        this.fButAddType.setFont(font);
        setButtonLayoutData(this.fButAddType);
        this.fButAddType.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                addType();
            }
        });

        // Add package button
        this.fButAddPackage = new Button(buttonContainer, SWT.PUSH);
        this.fButAddPackage.setText("Add Package");
        this.fButAddPackage.setToolTipText("Add Package");
        this.fButAddPackage.setFont(font);
        setButtonLayoutData(this.fButAddPackage);
        this.fButAddPackage.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                addPackage();
            }
        });

        // Remove button
        this.fButRemoveFilter = new Button(buttonContainer, SWT.PUSH);
        this.fButRemoveFilter.setText("Remove Filter");
        this.fButRemoveFilter.setToolTipText("Remove Filter");
        this.fButRemoveFilter.setFont(font);
        setButtonLayoutData(this.fButRemoveFilter);
        this.fButRemoveFilter.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                removeFilters();
            }
        });
        this.fButRemoveFilter.setEnabled(false);

        // copied from ListDialogField.CreateSeparator()
        Label separator = new Label(buttonContainer, SWT.NONE);
        separator.setVisible(false);
        gd = new GridData();
        gd.horizontalAlignment = GridData.FILL;
        gd.verticalAlignment = GridData.BEGINNING;
        gd.heightHint = 4;
        separator.setLayoutData(gd);

        this.fButEnableAll = new Button(buttonContainer, SWT.PUSH);
        this.fButEnableAll.setText("Enable all");
        this.fButEnableAll.setToolTipText("Enable all");
        this.fButEnableAll.setFont(font);
        setButtonLayoutData(this.fButEnableAll);
        this.fButEnableAll.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                checkAllFilters(true);
            }
        });

        this.fButDisableAll = new Button(buttonContainer, SWT.PUSH);
        this.fButDisableAll.setText("Disable all");
        this.fButDisableAll.setToolTipText("Disable all");
        this.fButDisableAll.setFont(font);
        setButtonLayoutData(this.fButDisableAll);
        this.fButDisableAll.addListener(SWT.Selection, new Listener() {
            public void handleEvent(Event e) {
                checkAllFilters(false);
            }
        });

    }

    /**
     * Create a group to contain the step filter related widgetry
     */
    private void createExtractionFilterPreferences(Composite parent) {
        Font font = parent.getFont();

        // top level container
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        layout.marginHeight = 0;
        layout.marginWidth = 0;
        container.setLayout(layout);
        GridData gd = new GridData(GridData.FILL_BOTH);
        container.setLayoutData(gd);
        container.setFont(font);

        //table label
        this.fLblTable = new Label(container, SWT.NONE);
        this.fLblTable.setText("Define Graph-Extraction Filter:");
        gd = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
        gd.horizontalSpan = 2;
        this.fLblTable.setLayoutData(gd);
        this.fLblTable.setFont(font);

        this.fTabViewFilter = CheckboxTableViewer.newCheckList(container,
                SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.MULTI
                        | SWT.FULL_SELECTION);
        this.fTabViewFilter.setLabelProvider(new FilterLabelProvider());
        this.fTabViewFilter.setSorter(new FilterViewerSorter());
        this.fFilterContentProvider = new FilterContentProvider(
                this.fTabViewFilter);
        this.fTabViewFilter.setContentProvider(this.fFilterContentProvider);
        // input just needs to be non-null
        this.fTabViewFilter.setInput(this);

        // filter table
        this.fTabFilter = this.fTabViewFilter.getTable();
        this.fTabFilter.setFont(font);
        gd = new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL);
        gd.widthHint = 1; // just set it to something small and let ig "GRAB"
        // the rest.
        this.fTabFilter.setLayoutData(gd);

        TableLayout tableLayout = new TableLayout();
        ColumnWeightData columnLayoutData = new ColumnWeightData(100);
        columnLayoutData.resizable = true;
        tableLayout.addColumnData(columnLayoutData);
        this.fTabFilter.setLayout(tableLayout);

        this.fTabViewFilter.addCheckStateListener(new ICheckStateListener() {
            public void checkStateChanged(CheckStateChangedEvent event) {
                Filter filter = (Filter) event.getElement();
                GraphExtractorPreferencePage.this.fFilterContentProvider
                        .toggleFilter(filter);
            }
        });
        this.fTabViewFilter
                .addSelectionChangedListener(new ISelectionChangedListener() {
                    public void selectionChanged(SelectionChangedEvent event) {
                        ISelection selection = event.getSelection();
                        if (selection.isEmpty()) {
                            GraphExtractorPreferencePage.this.fButRemoveFilter
                                    .setEnabled(false);
                        } else {
                            GraphExtractorPreferencePage.this.fButRemoveFilter
                                    .setEnabled(true);
                        }
                    }
                });
        this.fTabViewFilter.getControl().addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent event) {
                //				handleFilterViewerKeyPress(event);
            }
        });

        createExtractionFilterButtons(container);
    }

    void addPackage() {
        Shell shell = getShell();
        ElementListSelectionDialog dialog = null;
        try {
            dialog = JDIDebugUIPlugin.createAllPackagesDialog(shell, null,
                    false);
        } catch (JavaModelException jme) {
            String title = "Add packages to graph extractor filter";
            String message = "Select a package to filter when graph extraction";
            ExceptionHandler.handle(jme, title, message);
            return;
        }

        if (dialog == null) {
            return;
        }

        dialog.setTitle("Add packages to graph extractor filter");
        dialog.setMessage("Select a package to filter when graph extraction");
        dialog.setMultipleSelection(true);
        if (dialog.open() == IDialogConstants.CANCEL_ID) {
            return;
        }

        Object[] packages = dialog.getResult();
        if (packages != null) {
            for (int i = 0; i < packages.length; i++) {
                IJavaElement pkg = (IJavaElement) packages[i];

                String filter = pkg.getElementName() + ".*"; //$NON-NLS-1$
                this.fFilterContentProvider.addFilter(filter, true);
            }
        }
    }

    /**
     * Content provider for the table. Content consists of instances of
     * ExtractionFilter.
     */
    protected class FilterContentProvider implements IStructuredContentProvider {

        private CheckboxTableViewer fViewer;
        private List fFilters;

        public FilterContentProvider(CheckboxTableViewer viewer) {
            this.fViewer = viewer;
            List active = createActiveExtractionFiltersList();
            List inactive = createInactiveExtractionFiltersList();
            populateFilters(active, inactive);
            updateActions();
        }

        public void setDefaults() {
            this.fViewer.remove(this.fFilters.toArray());
            List active = createDefaultActiveExtractionFiltersList();
            List inactive = createDefaultInactiveExtractionFiltersList();
            populateFilters(active, inactive);
        }

        protected void populateFilters(List activeList, List inactiveList) {
            this.fFilters = new ArrayList(activeList.size()
                    + inactiveList.size());
            populateList(activeList, true);
            populateList(inactiveList, false);
        }

        protected void populateList(List list, boolean checked) {
            Iterator iterator = list.iterator();
            while (iterator.hasNext()) {
                String name = (String) iterator.next();
                addFilter(name, checked);
            }
        }

        public Filter addFilter(String name, boolean checked) {
            Filter filter = new Filter(name, checked);
            if (!this.fFilters.contains(filter)) {
                this.fFilters.add(filter);
                this.fViewer.add(filter);
                this.fViewer.setChecked(filter, checked);
            }
            updateActions();
            return filter;
        }

        public void saveFilters() {

            List active = new ArrayList(this.fFilters.size());
            List inactive = new ArrayList(this.fFilters.size());
            Iterator iterator = this.fFilters.iterator();
            while (iterator.hasNext()) {
                Filter filter = (Filter) iterator.next();
                String name = filter.getName();
                if (filter.isChecked()) {
                    active.add(name);
                } else {
                    inactive.add(name);
                }
            }
            String pref = JavaDebugOptionsManager
                    .serializeList((String[]) active.toArray(new String[active
                            .size()]));
            getPreferenceStore()
                    .setValue(
                            IDDStatePreferenceConstants.GRAPHEXTRACTOR_ACTIVE_FILTERS_LIST,
                            pref);
            pref = JavaDebugOptionsManager.serializeList((String[]) inactive
                    .toArray(new String[inactive.size()]));
            getPreferenceStore()
                    .setValue(
                            IDDStatePreferenceConstants.GRAPHEXTRACTOR_INACTIVE_FILTERS_LIST,
                            pref);
        }

        public void removeFilters(Object[] filters) {
            for (int i = 0; i < filters.length; i++) {
                Filter filter = (Filter) filters[i];
                this.fFilters.remove(filter);
            }
            this.fViewer.remove(filters);
            updateActions();
        }

        public void setChecked(Filter filter, boolean checked) {
            filter.setChecked(checked);
            this.fViewer.setChecked(filter, checked);
        }

        public void toggleFilter(Filter filter) {
            boolean newState = !filter.isChecked();
            filter.setChecked(newState);
            this.fViewer.setChecked(filter, newState);
        }

        /**
         * @see IStructuredContentProvider#getElements(Object)
         */
        public Object[] getElements(Object inputElement) {
            return this.fFilters.toArray(new Filter[0]);
        }

        /**
         * @see IContentProvider#inputChanged(Viewer, Object, Object)
         */
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
            // do nothing
        }

        /**
         * @see IContentProvider#dispose()
         */
        public void dispose() {
            // do nothing
        }
    }

    /**
     * Returns a list of active step filters.
     * 
     * @return list
     */
    protected List createActiveExtractionFiltersList() {
        String[] strings = JavaDebugOptionsManager
                .parseList(getPreferenceStore()
                        .getString(
                                IDDStatePreferenceConstants.GRAPHEXTRACTOR_ACTIVE_FILTERS_LIST));
        return Arrays.asList(strings);
    }

    /**
     * Returns a list of active step filters.
     * 
     * @return list
     */
    protected List createInactiveExtractionFiltersList() {
        String[] strings = JavaDebugOptionsManager
                .parseList(getPreferenceStore()
                        .getString(
                                IDDStatePreferenceConstants.GRAPHEXTRACTOR_INACTIVE_FILTERS_LIST));
        return Arrays.asList(strings);
    }

    /**
     * Returns a list of the default active step filters.
     * 
     * @return list
     */
    protected List createDefaultActiveExtractionFiltersList() {
        String[] strings = JavaDebugOptionsManager
                .parseList(getPreferenceStore()
                        .getDefaultString(
                                IDDStatePreferenceConstants.GRAPHEXTRACTOR_ACTIVE_FILTERS_LIST));
        return Arrays.asList(strings);
    }

    /**
     * Returns a list of the default active step filters.
     * 
     * @return list
     */
    protected List createDefaultInactiveExtractionFiltersList() {
        String[] strings = JavaDebugOptionsManager
                .parseList(getPreferenceStore()
                        .getDefaultString(
                                IDDStatePreferenceConstants.GRAPHEXTRACTOR_INACTIVE_FILTERS_LIST));
        return Arrays.asList(strings);
    }

    protected void updateActions() {
        if (this.fEnableAllButton != null) {
            boolean enabled = this.fTabViewFilter.getTable().getItemCount() > 0;
            this.fEnableAllButton.setEnabled(enabled);
            this.fDisableAllButton.setEnabled(enabled);
        }
    }

    void checkAllFilters(boolean check) {

        Object[] filters = this.fFilterContentProvider.getElements(null);
        for (int i = 0; i != filters.length; i++) {
            ((Filter) filters[i]).setChecked(check);
        }

        this.fTabViewFilter.setAllChecked(check);
    }

    /**
     * Removes the selected filters.
     */
    void removeFilters() {
        IStructuredSelection selection = (IStructuredSelection) this.fTabViewFilter
                .getSelection();
        this.fFilterContentProvider.removeFilters(selection.toArray());
    }

    void addType() {
        Shell shell = getShell();
        SelectionDialog dialog = null;
        try {
            dialog = JavaUI.createTypeDialog(shell, PlatformUI.getWorkbench()
                    .getProgressService(), SearchEngine.createWorkspaceScope(),
                    IJavaElementSearchConstants.CONSIDER_CLASSES, false);
        } catch (JavaModelException jme) {
            String title = "Add type to graph extractor filter";
            String message = "Select a type to filter when graph extraction";
            ExceptionHandler.handle(jme, title, message);
            return;
        }

        dialog.setTitle("Add type to graph extractor filter");
        dialog.setMessage("Select a type to filter when graph extraction");
        if (dialog.open() == IDialogConstants.CANCEL_ID) {
            return;
        }

        Object[] types = dialog.getResult();
        if (types != null && types.length > 0) {
            IType type = (IType) types[0];
            this.fFilterContentProvider.addFilter(type.getFullyQualifiedName(),
                    true);
        }
    }

    void addFilter() {
            	Shell shell = getShell();

        Filter[] existingFilters = (Filter[]) this.fFilterContentProvider
                .getElements(null);
        Filter newFilter = CreateStepFilterDialog.showCreateStepFilterDialog(
                shell, existingFilters);
        if (newFilter != null)
            commitNewFilter(newFilter);
    }

    private void commitNewFilter(Filter newFilter) {
        String newFilterName = newFilter.getName();

        // if newFilter is a duplicate of existing, just set state of existing
        // to checked.
        Object[] filters = this.fFilterContentProvider.getElements(null);
        for (int i = 0; i < filters.length; i++) {
            Filter filter = (Filter) filters[i];
            if (filter.getName().equals(newFilterName)) {
                this.fFilterContentProvider.setChecked(filter, true);
                return;
            }
        }
        this.fFilterContentProvider.addFilter(newFilterName, true);
        this.fTabItemNew = this.fTabFilter.getItem(0);
        this.fTabItemNew.setText(newFilterName);
        this.fTabViewFilter.refresh();
    }

}