/**
 * <p>Title: Bytecode instrumentation</p>
 * <p>Description: Tracing framework for java bytecode</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Universitt Saarbrcken</p>
 * @author Valentin Dallmeier
 * @version 1.0
 */
package org.softevo.sibrelib.source;

/** This class groups the bytecode representation
 *  of a class together with the class name. <BR>
 *  It also supports storage of a modified version.
 */
public class ClassFile {

  /** The name of the class. <BR>
   */
  private String className = null;

  /** The bytecode representation. <BR>
   */
  private byte[] classBytes = null;

  /** The modified version of the class file. <BR>
   */
  private byte[] modifiedClassBytes = null;


  /** Creates a new class file. <BR>
   *  @param className the name of the class
   *  @param classBytes the bytecode representation of the class
   */
  public ClassFile(String className, byte[] classBytes) {
    this.className = className;
    this.classBytes = classBytes;
  }

  /** Tests wether or not this class file
   *  was modified. <BR>
   *  @return <code>true</code> if the class file was modified,
   *          <code>false</code> otherwise
   */
  public boolean isModified() {
    return this.modifiedClassBytes != null;
  }

  /** Sets the bytes of the modified version. <BR>
   *  @param modifiedClassBytes the bytecode of the modified version
   */
  public void setModifiedClassBytes(byte[] modifiedClassBytes) {
    this.modifiedClassBytes = modifiedClassBytes;
  }

  /** Gets the name of the class. <BR>
   *  @return the fully qualified class name
   */
  public String getClassName() {
    return this.className;
  }

  /** Gets the original (unmodified) version of the class. <BR>
   *  @return the class bytes
   */
  public byte[] getClassBytes() {
    return this.classBytes;
  }

  /** Gets the bytes of the modified version. <BR>
   *  @return the modified class bytes or <code>null</code> if
   *  the class was not modified
   */
  public byte[] getModifiedClassBytes() {
    return this.modifiedClassBytes;
  }
}